@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      actions.cpp
-  Version:   $Rev: 1692 $
+  Version:   $Rev: 1977 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              30-Apr-06, Roger Larsson: Print filename if processing multiple files
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: actions.cpp 1692 2008-12-08 05:32:02Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: actions.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -48,6 +48,7 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 1692 2008-12-08 05:32:02Z ahuggel $")
 #include "utils.hpp"
 #include "types.hpp"
 #include "exif.hpp"
+#include "easyaccess.hpp"
 #include "canonmn.hpp"
 #include "iptc.hpp"
 #include "xmp.hpp"
@@ -127,6 +128,12 @@ namespace {
                  int targetType,
                  bool preserve);
 
+    /*!
+      @brief Test to distinguish between Exif and other tags in TIFF-like files.
+             Only Exif tags are copyed, deleted, added.
+     */
+    bool isExifTag(const Exiv2::Exifdatum& ed);
+
     /*!
       @brief Rename a file according to a timestamp value.
 
@@ -160,6 +167,10 @@ namespace {
 // class member definitions
 namespace Action {
 
+    Task::~Task()
+    {
+    }
+
     Task::AutoPtr Task::clone() const
     {
         return AutoPtr(clone_());
@@ -219,6 +230,10 @@ namespace Action {
         return Task::AutoPtr(0);
     } // TaskFactory::create
 
+    Print::~Print()
+    {
+    }
+
     int Print::run(const std::string& path)
     try {
         path_ = path;
@@ -286,7 +301,7 @@ namespace Action {
 
         // Image number
         // Todo: Image number for cameras other than Canon
-        printTag(exifData, "Exif.Canon.ImageNumber", _("Image number"));
+        printTag(exifData, "Exif.Canon.FileNumber", _("Image number"));
 
         // Exposure time
         // From ExposureTime, failing that, try ShutterSpeedValue
@@ -318,23 +333,8 @@ namespace Action {
         // Flash
         printTag(exifData, "Exif.Photo.Flash", _("Flash"));
 
-        // Todo: Flash bias, flash energy
-        // Todo: Implement this for other cameras
-        done = false;
-        printLabel(_("Flash bias"));
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonSi.FlashBias");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Panasonic.FlashBias");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Olympus.FlashBias");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.OlympusCs.FlashExposureComp");
-        }
-        std::cout << std::endl;
+        // Flash bias
+        printTag(exifData, Exiv2::flashBias, _("Flash bias"));
 
         // Actual focal length and 35 mm equivalent
         // Todo: Calculate 35 mm equivalent a la jhead
@@ -348,6 +348,9 @@ namespace Action {
                           << md->print(&exifData) << ")";
             }
         }
+        else {
+            printTag(exifData, "Exif.Canon.FocalLength");
+        }
         std::cout << std::endl;
 
         // Subject distance
@@ -362,157 +365,43 @@ namespace Action {
         std::cout << std::endl;
 
         // ISO speed
-        // from ISOSpeedRatings or the Makernote
-        printLabel(_("ISO speed"));
-        done = false;
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Photo.ISOSpeedRatings");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonSi.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon1.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon2.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon3.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsNew.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsOld.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs5D.ISOSpeed");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs7D.ISOSpeed");
-        }
-        std::cout << std::endl;
+        printTag(exifData, Exiv2::isoSpeed, _("ISO speed"));
 
         // Exposure mode
-        // From ExposureProgram or Canon Makernote
-        printLabel(_("Exposure mode"));
-        done = false;
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Photo.ExposureProgram");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs.ExposureProgram");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs7D.ExposureMode");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs5D.ExposureMode");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsNew.ExposureMode");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsOld.ExposureMode");
-        }
-        std::cout << std::endl;
+        printTag(exifData, Exiv2::exposureMode, _("Exposure mode"));
 
         // Metering mode
         printTag(exifData, "Exif.Photo.MeteringMode", _("Metering mode"));
 
         // Macro mode
-        // Todo: Implement this for other cameras
-        printLabel(_("Macro mode"));
-        done = false;
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs.Macro");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Fujifilm.Macro");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Olympus.Macro");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.OlympusCs.MacroMode");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Panasonic.Macro");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsNew.MacroMode");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsOld.MacroMode");
-        }
-        std::cout << std::endl;
+        printTag(exifData, Exiv2::macroMode, _("Macro mode"));
 
         // Image quality setting (compression)
-        // Todo: Implement this for other cameras
-        printLabel(_("Image quality"));
-        done = false;
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Fujifilm.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Sigma.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon1.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon2.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon3.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Olympus.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.OlympusCs.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Panasonic.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Minolta.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsNew.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsOld.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs5D.Quality");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs7D.Quality");
-        }
-        std::cout << std::endl;
+        printTag(exifData, Exiv2::imageQuality, _("Image quality"));
 
         // Exif Resolution
         printLabel(_("Exif Resolution"));
         long xdim = 0;
         long ydim = 0;
-        md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"));
-        if (md == exifData.end()) {
-            md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+        if (image->mimeType() == "image/tiff") {
+            xdim = image->pixelWidth();
+            ydim = image->pixelHeight();
         }
-        if (md != exifData.end() && md->count() > 0) {
-            xdim = md->toLong();
-        }
-        md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
-        if (md == exifData.end()) {
-            md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
-        }
-        if (md != exifData.end() && md->count() > 0) {
-            ydim = md->toLong();
+        else {
+            md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"));
+            if (md == exifData.end()) {
+                md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+            }
+            if (md != exifData.end() && md->count() > 0) {
+                xdim = md->toLong();
+            }
+            md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
+            if (md == exifData.end()) {
+                md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
+            }
+            if (md != exifData.end() && md->count() > 0) {
+                ydim = md->toLong();
+            }
         }
         if (xdim != 0 && ydim != 0) {
             std::cout << xdim << " x " << ydim;
@@ -520,52 +409,7 @@ namespace Action {
         std::cout << std::endl;
 
         // White balance
-        // Todo: Implement this for other cameras
-        printLabel(_("White balance"));
-        done = false;
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonSi.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Fujifilm.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Sigma.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon1.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon2.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Nikon3.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Olympus.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.OlympusCs.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Panasonic.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs5D.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCs7D.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsNew.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.MinoltaCsOld.WhiteBalance");
-        }
-        if (!done) {
-            done = 0 != printTag(exifData, "Exif.Photo.WhiteBalance");
-        }
-        std::cout << std::endl;
+        printTag(exifData, Exiv2::whiteBalance, _("White balance"));
 
         // Thumbnail
         printLabel(_("Thumbnail"));
@@ -624,6 +468,23 @@ namespace Action {
         return rc;
     } // Print::printTag
 
+    int Print::printTag(const Exiv2::ExifData& exifData,
+                        EasyAccessFct easyAccessFct,
+                        const std::string& label) const
+    {
+        int rc = 0;
+        if (!label.empty()) {
+            printLabel(label);
+        }
+        Exiv2::ExifData::const_iterator md = easyAccessFct(exifData);
+        if (md != exifData.end()) {
+            md->write(std::cout, &exifData);
+            rc = 1;
+        }
+        if (!label.empty()) std::cout << std::endl;
+        return rc;
+    } // Print::printTag
+
     int Print::printList()
     {
         int rc = 0;
@@ -761,7 +622,9 @@ namespace Action {
             if (!first) std::cout << "  ";
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
                 && md.size() > 100) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
@@ -772,7 +635,9 @@ namespace Action {
             if (!first) std::cout << "  ";
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
                 && md.size() > 100) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
@@ -783,7 +648,9 @@ namespace Action {
             if (!first) std::cout << std::endl;
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
                 && md.size() > 100) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
@@ -852,6 +719,10 @@ namespace Action {
         return new Print(*this);
     }
 
+    Rename::~Rename()
+    {
+    }
+
     int Rename::run(const std::string& path)
     {
     try {
@@ -937,6 +808,10 @@ namespace Action {
         return new Rename(*this);
     }
 
+    Erase::~Erase()
+    {
+    }
+
     int Erase::run(const std::string& path)
     try {
         path_ = path;
@@ -1005,7 +880,13 @@ namespace Action {
         if (Params::instance().verbose_ && image->exifData().count() > 0) {
             std::cout << _("Erasing Exif data from the file") << std::endl;
         }
-        image->clearExifData();
+        if (0 == strcmp(image->mimeType().c_str(), "image/tiff")) {
+            Exiv2::ExifData& ed = image->exifData();
+            ed.erase(std::remove_if(ed.begin(), ed.end(), isExifTag), ed.end());
+        }
+        else {
+            image->clearExifData();
+        }
         return 0;
     }
 
@@ -1032,6 +913,7 @@ namespace Action {
         if (Params::instance().verbose_ && image->xmpData().count() > 0) {
             std::cout << _("Erasing XMP data from the file") << std::endl;
         }
+        image->clearXmpData();                  // Quick fix for bug #612
         image->clearXmpPacket();
         return 0;
     }
@@ -1046,6 +928,10 @@ namespace Action {
         return new Erase(*this);
     }
 
+    Extract::~Extract()
+    {
+    }
+
     int Extract::run(const std::string& path)
     try {
         path_ = path;
@@ -1184,6 +1070,10 @@ namespace Action {
         return new Extract(*this);
     }
 
+    Insert::~Insert()
+    {
+    }
+
     int Insert::run(const std::string& path)
     try {
         if (!Exiv2::fileExists(path, true)) {
@@ -1283,6 +1173,10 @@ namespace Action {
         return new Insert(*this);
     }
 
+    Modify::~Modify()
+    {
+    }
+
     int Modify::run(const std::string& path)
     {
     try {
@@ -1518,6 +1412,10 @@ namespace Action {
         return new Modify(*this);
     }
 
+    Adjust::~Adjust()
+    {
+    }
+
     int Adjust::run(const std::string& path)
     try {
         adjustment_      = Params::instance().adjustment_;
@@ -1654,6 +1552,10 @@ namespace Action {
         return 0;
     } // Adjust::adjustDateTime
 
+    FixIso::~FixIso()
+    {
+    }
+
     int FixIso::run(const std::string& path)
     {
     try {
@@ -1675,37 +1577,15 @@ namespace Action {
                       << ": " << _("No Exif data found in the file\n");
             return -3;
         }
-        // Check if the standard tag exists
-        Exiv2::ExifKey key("Exif.Photo.ISOSpeedRatings");
-        Exiv2::ExifData::iterator md = exifData.findKey(key);
+        Exiv2::ExifData::const_iterator md = Exiv2::isoSpeed(exifData);
         if (md != exifData.end()) {
-            if (Params::instance().verbose_) {
-                std::cout << _("Standard Exif ISO tag exists; not modified\n");
+            if (strcmp(md->key().c_str(), "Exif.Photo.ISOSpeedRatings") == 0) {
+                if (Params::instance().verbose_) {
+                    std::cout << _("Standard Exif ISO tag exists; not modified\n");
+                }
+                return 0;
             }
-            return 0;
-        }
-        // Fix Nikon ISO setting
-        key = Exiv2::ExifKey("Exif.Nikon3.ISOSpeed");
-        md = exifData.findKey(key);
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.Nikon2.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.Nikon1.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        // Canon has a similar bad habit, fix that too
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.CanonSi.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.CanonCs.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        // Copy the proprietary tag to the standard place
-        if (md != exifData.end()) {
+            // Copy the proprietary tag to the standard place
             std::ostringstream os;
             md->write(os, &exifData);
             if (Params::instance().verbose_) {
@@ -1859,7 +1739,22 @@ namespace {
                 std::cout << _("Writing Exif data from") << " " << source
                           << " " << _("to") << " " << target << std::endl;
             }
-            targetImage->setExifData(sourceImage->exifData());
+            if (0 == strcmp(targetImage->mimeType().c_str(), "image/tiff")) {
+                Exiv2::ExifData& ted = targetImage->exifData();
+                if (!preserve) {
+                    targetImage->readMetadata();
+                    ted.erase(std::remove_if(ted.begin(), ted.end(), isExifTag), ted.end());
+                }
+                const Exiv2::ExifData& sed = sourceImage->exifData();
+                for (Exiv2::ExifData::const_iterator pos = sed.begin(); pos != sed.end(); ++pos) {
+                    if (isExifTag(*pos)) {
+                        ted[pos->key()] = pos->value();
+                    }
+                }
+            }
+            else {
+                targetImage->setExifData(sourceImage->exifData());
+            }
         }
         if (   Params::instance().target_ & Params::ctIptc
             && !sourceImage->iptcData().empty()) {
@@ -1897,6 +1792,74 @@ namespace {
         return 0;
     } // metacopy
 
+    // Defined outside of the function so that Exiv2::find() can see it
+    struct String {
+        const char* s_;
+        bool operator==(const char* s) const {
+            return 0 == strcmp(s_, s);
+        }
+    };
+
+    bool isExifTag(const Exiv2::Exifdatum& ed)
+    {
+        // A somewhat random list of IFD0 tags which are considered as "Exif tags"
+        static const String exifTags[] = {
+            { "Exif.Image.ProcessingSoftware"      },
+            { "Exif.Image.DocumentName"            },
+            { "Exif.Image.ImageDescription"        },
+            { "Exif.Image.Make"                    },
+            { "Exif.Image.Model"                   },
+            { "Exif.Image.Software"                },
+            { "Exif.Image.DateTime"                },
+            { "Exif.Image.HostComputer"            },
+            { "Exif.Image.Artist"                  },
+            { "Exif.Image.XMLPacket"               },
+            { "Exif.Image.Rating"                  },
+            { "Exif.Image.RatingPercent"           },
+            { "Exif.Image.CFARepeatPatternDim"     },
+            { "Exif.Image.CFAPattern"              },
+            { "Exif.Image.BatteryLevel"            },
+            { "Exif.Image.IPTCNAA"                 },
+            { "Exif.Image.Copyright"               },
+            { "Exif.Image.ImageResources"          },
+            { "Exif.Image.ExifTag"                 },
+            { "Exif.Image.InterColorProfile"       },
+            { "Exif.Image.GPSTag"                  },
+            { "Exif.Image.XPTitle"                 },
+            { "Exif.Image.XPComment"               },
+            { "Exif.Image.XPAuthor"                },
+            { "Exif.Image.XPKeywords"              },
+            { "Exif.Image.XPSubject"               },
+            { "Exif.Image.PrintImageMatching"      },
+            { "Exif.Image.UniqueCameraModel"       },
+            { "Exif.Image.LocalizedCameraModel"    },
+            { "Exif.Image.CFAPlaneColor"           },
+            { "Exif.Image.CFALayout"               },
+            { "Exif.Image.CameraSerialNumber"      },
+            { "Exif.Image.LensInfo"                },
+            { "Exif.Image.OriginalRawFileName"     },
+            { "Exif.Image.ActiveArea"              },
+            { "Exif.Image.MaskedAreas"             },
+            { "Exif.Image.AsShotICCProfile"        },
+            { "Exif.Image.AsShotPreProfileMatrix"  },
+            { "Exif.Image.CurrentICCProfile"       },
+            { "Exif.Image.CurrentPreProfileMatrix" }
+        };
+
+        static const Exiv2::IfdId exifIfds[] = {
+            Exiv2::exifIfdId,
+            Exiv2::gpsIfdId,
+            Exiv2::iopIfdId
+        };
+
+        if (   0 != Exiv2::find(exifIfds, ed.ifdId())
+            || Exiv2::ExifTags::isMakerIfd(ed.ifdId())
+            || 0 != Exiv2::find(exifTags, ed.key().c_str())) {
+            return true;
+        }
+        return false;
+    } // isExifTag
+
     int renameFile(std::string& newPath, const struct tm* tm)
     {
         std::string path = newPath;
