@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    basicio.hpp
   @brief   Simple binary IO abstraction
-  @version $Rev: 1703 $
+  @version $Rev: 1973 $
   @author  Brad Schick (brad)
            <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
   @date    04-Dec-04, brad: created
@@ -35,8 +35,7 @@
 
 // + standard includes
 #include <string>
-#include <memory>
-#include <cstdio>
+#include <memory>                               // for std::auto_ptr
 
 // *****************************************************************************
 // namespace extensions
@@ -63,7 +62,7 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Destructor
-        virtual ~BasicIo() {}
+        virtual ~BasicIo();
         //@}
 
         //! @name Manipulators
@@ -173,17 +172,21 @@ namespace Exiv2 {
         virtual int seek(long offset, Position pos) = 0;
         /*!
           @brief Direct access to the IO data. For files, this is done by
-                 mapping the file into the process's address space; for
-                 memory blocks, this allows direct access to the memory
-                 block.
-
-          Todo: This is currently only for read access.
+                 mapping the file into the process's address space; for memory
+                 blocks, this allows direct access to the memory block.
+          @param isWriteable Set to true if the mapped area should be writeable
+                 (default is false).
+          @return A pointer to the mapped area.
+          @throw Error In case of failure.
          */
-        virtual const byte* mmap() =0;
+        virtual byte* mmap(bool isWriteable =false) =0;
         /*!
-          @brief Remove a mapping established with mmap().
+          @brief Remove a mapping established with mmap(). If the mapped area
+                 is writeable, this ensures that changes are written back.
+          @return 0 if successful;<BR>
+                  Nonzero if failure;
          */
-        virtual void  munmap() =0;
+        virtual int munmap() =0;
         //@}
 
         //! @name Accessors
@@ -212,6 +215,13 @@ namespace Exiv2 {
               available.
          */
         virtual std::string path() const =0;
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like path() but returns a unicode path in an std::wstring.
+          @note This function is only available on Windows.
+         */
+        virtual std::wstring wpath() const =0;
+#endif
         /*!
           @brief Returns a temporary data storage location. This is often
               needed to rewrite an IO source.
@@ -247,7 +257,7 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Constructor, takes a BasicIo reference
-        IoCloser(BasicIo &bio) : bio_(bio) {}
+        IoCloser(BasicIo& bio) : bio_(bio) {}
         //! Destructor, closes the BasicIo reference
         ~IoCloser() { close(); }
         //@}
@@ -270,7 +280,6 @@ namespace Exiv2 {
         IoCloser& operator=(const IoCloser&);
     }; // class IoCloser
 
-
     /*!
       @brief Provides binary file IO by implementing the BasicIo
           interface.
@@ -286,6 +295,14 @@ namespace Exiv2 {
           @param path The full path of a file
          */
         FileIo(const std::string& path);
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like FileIo(const std::string& path) but accepts a
+              unicode path in an std::wstring.
+          @note This constructor is only available on Windows.
+         */
+        FileIo(const std::wstring& wpath);
+#endif
         //! Destructor. Flushes and closes an open file.
         virtual ~FileIo();
         //@}
@@ -409,13 +426,25 @@ namespace Exiv2 {
          */
         virtual int seek(long offset, Position pos);
         /*!
-          @brief Map the file into the process's address space. The file must
-                 be open before mmap() is called.
-          @note  The returned pointer is valid only as long as the MemIo object
-                 is in scope.
+          @brief Map the file into the process's address space. The file must be
+                 open before mmap() is called. If the mapped area is writeable,
+                 changes may not be written back to the underlying file until
+                 munmap() is called. The pointer is valid only as long as the
+                 FileIo object exists.
+          @param isWriteable Set to true if the mapped area should be writeable
+                 (default is false).
+          @return A pointer to the mapped area.
+          @throw Error In case of failure.
          */
-        virtual const byte* mmap();
-        virtual void  munmap();
+        virtual byte* mmap(bool isWriteable =false);
+        /*!
+          @brief Remove a mapping established with mmap(). If the mapped area is
+                 writeable, this ensures that changes are written back to the
+                 underlying file.
+          @return 0 if successful;<BR>
+                  Nonzero if failure;
+         */
+        virtual int munmap();
         //@}
 
         //! @name Accessors
@@ -441,6 +470,13 @@ namespace Exiv2 {
         virtual bool eof() const;
         //! Returns the path of the file
         virtual std::string path() const;
+#ifdef EXV_UNICODE_PATH
+        /*
+          @brief Like path() but returns the unicode path of the file in an std::wstring.
+          @note This function is only available on Windows.
+         */
+        virtual std::wstring wpath() const;
+#endif
         /*!
           @brief Returns a temporary data storage location. The actual type
               returned depends upon the size of the file represented a FileIo
@@ -460,27 +496,9 @@ namespace Exiv2 {
         //! Assignment operator
         FileIo& operator=(const FileIo& rhs);
 
-        // Enumeration
-        enum OpMode { opRead, opWrite, opSeek };
-
-        // DATA
-        std::string path_;
-        std::string openMode_;
-        FILE *fp_;
-        OpMode opMode_;
-
-        byte* pMappedArea_;
-        size_t mappedLength_;
-        bool isMalloced_;               //!< Is the mapped area allocated?
-
-        // METHODS
-        /*!
-          @brief Switch to a new access mode, reopening the file if needed.
-              Optimized to only reopen the file when it is really necessary.
-          @param opMode The mode to switch to.
-          @return 0 if successful
-         */
-        EXV_DLLLOCAL int switchMode(OpMode opMode);
+        // Pimpl idiom
+        class Impl;
+        Impl* p_;
 
     }; // class FileIo
 
@@ -617,14 +635,14 @@ namespace Exiv2 {
         virtual int seek(long offset, Position pos);
         /*!
           @brief Allow direct access to the underlying data buffer. The buffer
-                 is not protected against write access except for the const
-                 specifier.
+                 is not protected against write access in any way, the argument
+                 is ignored.
           @note  The application must ensure that the memory pointed to by the
                  returned pointer remains valid and allocated as long as the
-                 MemIo object is in scope.
+                 MemIo object exists.
          */
-        virtual const byte* mmap() { return data_; }
-        virtual void  munmap() {}
+        virtual byte* mmap(bool /*isWriteable*/ =false);
+        virtual int munmap();
         //@}
 
         //! @name Accessors
@@ -648,6 +666,13 @@ namespace Exiv2 {
         virtual bool eof() const;
         //! Returns a dummy path, indicating that memory access is used
         virtual std::string path() const;
+#ifdef EXV_UNICODE_PATH
+        /*
+          @brief Like path() but returns a unicode dummy path in an std::wstring.
+          @note This function is only available on Windows.
+         */
+        virtual std::wstring wpath() const;
+#endif
         /*!
           @brief Returns a temporary data storage location. Currently returns
               an empty MemIo object, but callers should not rely on this
@@ -664,26 +689,42 @@ namespace Exiv2 {
         //! Assignment operator
         MemIo& operator=(const MemIo& rhs);
 
-        // DATA
-        byte* data_;
-        long idx_;
-        long size_;
-        long sizeAlloced_;              //!< Size of the allocated buffer
-        bool isMalloced_;               //!< Was the buffer allocated?
-        bool eof_;
+        // Pimpl idiom
+        class Impl;
+        Impl* p_;
 
-        // METHODS
-        EXV_DLLLOCAL void reserve(long wcount);
     }; // class MemIo
 
 // *****************************************************************************
 // template, inline and free functions
 
-    //! Read file \em path into a DataBuf, which is returned.
+    /*!
+      @brief Read file \em path into a DataBuf, which is returned.
+      @return Buffer containing the file.
+      @throw Error In case of failure.
+     */
     EXIV2API DataBuf readFile(const std::string& path);
-
-    //! Write DataBuf \em buf to file \em path. Return the number of bytes written.
+#ifdef EXV_UNICODE_PATH
+    /*!
+      @brief Like readFile() but accepts a unicode path in an std::wstring.
+      @note This function is only available on Windows.
+     */
+    EXIV2API DataBuf readFile(const std::wstring& wpath);
+#endif
+    /*!
+      @brief Write DataBuf \em buf to file \em path.
+      @return Return the number of bytes written.
+      @throw Error In case of failure.
+     */
     EXIV2API long writeFile(const DataBuf& buf, const std::string& path);
+#ifdef EXV_UNICODE_PATH
+    /*!
+      @brief Like writeFile() but accepts a unicode path in an std::wstring.
+      @note This function is only available on Windows.
+     */
+    EXIV2API long writeFile(const DataBuf& buf, const std::wstring& wpath);
+
+#endif
 
 }                                       // namespace Exiv2
 
