@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      bmpimage.cpp
-  Version:   $Rev: 1703 $
+  Version:   $Rev: 1937 $
   Author(s): Marco Piovanelli, Ovolab (marco)
   History:   05-Mar-2007, marco: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: bmpimage.cpp 1703 2008-12-15 16:24:37Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: bmpimage.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 //#define DEBUG 1
 
@@ -57,6 +57,11 @@ namespace Exiv2 {
     {
     } // BmpImage::BmpImage
 
+    std::string BmpImage::mimeType() const
+    {
+        return "image/x-ms-bmp";
+    }
+
     void BmpImage::setExifData(const ExifData& /*exifData*/)
     {
         // Todo: implement me!
