@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -19,16 +19,15 @@
  * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
-  File   : canonmn.cpp
-  Version: $Rev: 1668 $
-  Authors: Andreas Huggel (ahu) <ahuggel@gmx.net>
-           David Cannings (dc) <david@edeca.net>
-  Credits: EXIF MakerNote of Canon by David Burren <http://www.burren.cx/david/canon.html>
-           Canon makernote tags by Phil Harvey <http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/Canon.html>
+  File:      canonmn.cpp
+  Version:   $Rev: 1944 $
+  Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
+             David Cannings (dc) <david@edeca.net>
+             Andi Clemens (ac) <andi.clemens@gmx.net>
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: canonmn.cpp 1668 2008-11-05 15:23:49Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: canonmn.cpp 1944 2009-11-28 08:28:08Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -165,13 +164,13 @@ namespace Exiv2 {
     const TagInfo CanonMakerNote::tagInfo_[] = {
         TagInfo(0x0000, "0x0000", "0x0000", N_("Unknown"), canonIfdId, makerTags, unsignedShort, printValue),
         TagInfo(0x0001, "CameraSettings", N_("Camera Settings"), N_("Various camera settings"), canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0002, "0x0002", "0x0002", N_("Unknown"), canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0002, "FocalLength", N_("Focal Length"), N_("Focal length"), canonIfdId, makerTags, unsignedShort, printFocalLength),
         TagInfo(0x0003, "0x0003", "0x0003", N_("Unknown"), canonIfdId, makerTags, unsignedShort, printValue),
         TagInfo(0x0004, "ShotInfo", N_("Shot Info"), N_("Shot information"), canonIfdId, makerTags, unsignedShort, printValue),
         TagInfo(0x0005, "Panorama", N_("Panorama"), N_("Panorama"), canonIfdId, makerTags, unsignedShort, printValue),
         TagInfo(0x0006, "ImageType", N_("Image Type"), N_("Image type"), canonIfdId, makerTags, asciiString, printValue),
         TagInfo(0x0007, "FirmwareVersion", N_("Firmware Version"), N_("Firmware version"), canonIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0008, "ImageNumber", N_("Image Number"), N_("Image number"), canonIfdId, makerTags, unsignedLong, print0x0008),
+        TagInfo(0x0008, "FileNumber", N_("File Number"), N_("File number"), canonIfdId, makerTags, unsignedLong, print0x0008),
         TagInfo(0x0009, "OwnerName", N_("Owner Name"), N_("Owner Name"), canonIfdId, makerTags, asciiString, printValue),
         TagInfo(0x000c, "SerialNumber", N_("Serial Number"), N_("Camera serial number"), canonIfdId, makerTags, unsignedLong, print0x000c),
         TagInfo(0x000d, "0x000d", "0x000d", N_("Unknown"), canonIfdId, makerTags, unsignedShort, printValue),
@@ -199,11 +198,12 @@ namespace Exiv2 {
 
     //! Quality, tag 0x0003
     extern const TagDetails canonCsQuality[] = {
-        { 1, N_("Economy")   },
-        { 2, N_("Normal")    },
-        { 3, N_("Fine")      },
-        { 4, N_("RAW")       },
-        { 5, N_("Superfine") }
+        { 1,   N_("Economy")      },
+        { 2,   N_("Normal")       },
+        { 3,   N_("Fine")         },
+        { 4,   N_("RAW")          },
+        { 5,   N_("Superfine")    },
+        { 130, N_("Normal Movie") }
     };
 
     //! FlashMode, tag 0x0004
@@ -376,6 +376,7 @@ namespace Exiv2 {
         {   6, "Sigma 18-50mm f/3.5-5.6 DC"                                 }, // 1
         {   6, "Sigma 18-125mm f/3.5-5.6 DC IF ASP"                         }, // 2
         {   6, "Tokina AF193-2 19-35mm f/3.5-4.5"                           }, // 3
+        {   6, "Sigma 28-80mm f/3.5-5.6 II Macro"                           }, // 4
         {   7, "Canon EF 100-300mm f/5.6L"                                  },
         {   8, "Canon EF 100-300mm f/5.6"                                   }, // 0
         {   8, "Sigma 70-300mm f/4-5.6 DG Macro"                            }, // 1
@@ -875,6 +876,150 @@ namespace Exiv2 {
         return tagInfoPi_;
     }
 
+    //! BracketMode, tag 0x0003
+    extern const TagDetails canonBracketMode[] = {
+        { 0, N_("Off") },
+        { 1, N_("AEB") },
+        { 2, N_("FEB") },
+        { 3, N_("ISO") },
+        { 4, N_("WB")  }
+    };
+
+    //! RawJpgSize, tag 0x0007
+    extern const TagDetails canonRawJpgSize[] = {
+        { 0,   N_("Large")        },
+        { 1,   N_("Medium")       },
+        { 2,   N_("Small")        },
+        { 5,   N_("Medium 1")     },
+        { 6,   N_("Medium 2")     },
+        { 7,   N_("Medium 3")     },
+        { 8,   N_("Postcard")     },
+        { 9,   N_("Widescreen")   },
+        { 129, N_("Medium Movie") },
+        { 130, N_("Small Movie")  }
+    };
+
+    //! NoiseReduction, tag 0x0008
+    extern const TagDetails canonNoiseReduction[] = {
+        { 0, N_("Off")  },
+        { 1, N_("On 1") },
+        { 2, N_("On 2") },
+        { 3, N_("On")   },
+        { 4, N_("Auto") }
+    };
+
+    //! WBBracketMode, tag 0x0009
+    extern const TagDetails canonWBBracketMode[] = {
+        { 0, N_("Off")           },
+        { 1, N_("On (shift AB)") },
+        { 2, N_("On (shift GM)") }
+    };
+
+    //! FilterEffect, tag 0x000e
+    extern const TagDetails canonFilterEffect[] = {
+        { 0, N_("None")   },
+        { 1, N_("Yellow") },
+        { 2, N_("Orange") },
+        { 3, N_("Red")    },
+        { 4, N_("Green")  }
+    };
+
+    //! ToningEffect, tag 0x000e
+    extern const TagDetails canonToningEffect[] = {
+        { 0, N_("None")   },
+        { 1, N_("Sepia")  },
+        { 2, N_("Blue")   },
+        { 3, N_("Purple") },
+        { 4, N_("Green")  }
+    };
+
+    // Canon File Info Tag
+    const TagInfo CanonMakerNote::tagInfoFi_[] = {
+        TagInfo(0x0001, "FileNumber", N_("File Number"), N_("File Number"), canonFiIfdId, makerTags, unsignedLong, printFiFileNumber),
+        TagInfo(0x0003, "BracketMode", N_("Bracket Mode"), N_("Bracket Mode"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonBracketMode)),
+        TagInfo(0x0004, "BracketValue", N_("Bracket Value"), N_("Bracket Value"), canonFiIfdId, makerTags, signedShort, printValue),
+        TagInfo(0x0005, "BracketShotNumber", N_("Bracket Shot Number"), N_("Bracket Shot Number"), canonFiIfdId, makerTags, signedShort,  printValue),
+        TagInfo(0x0006, "RawJpgQuality", N_("Raw Jpg Quality"), N_("Raw Jpg Quality"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonCsQuality)),
+        TagInfo(0x0007, "RawJpgSize", N_("Raw Jpg Size"), N_("Raw Jpg Size"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonRawJpgSize)),
+        TagInfo(0x0008, "NoiseReduction", N_("Noise Reduction"), N_("Noise Reduction"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonNoiseReduction)),
+        TagInfo(0x0009, "WBBracketMode", N_("WB Bracket Mode"), N_("WB Bracket Mode"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonWBBracketMode)),
+        TagInfo(0x000c, "WBBracketValueAB", N_("WB Bracket Value AB"), N_("WB Bracket Value AB"), canonFiIfdId, makerTags, signedShort, printValue),
+        TagInfo(0x000d, "WBBracketValueGM", N_("WB Bracket Value GM"), N_("WB Bracket Value GM"), canonFiIfdId, makerTags, signedShort, printValue),
+        TagInfo(0x000e, "FilterEffect", N_("Filter Effect"), N_("Filter Effect"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonFilterEffect)),
+        TagInfo(0x000f, "ToningEffect", N_("Toning Effect"), N_("Toning Effect"), canonFiIfdId, makerTags, signedShort, EXV_PRINT_TAG(canonToningEffect)),
+        // End of list marker
+        TagInfo(0xffff, "(UnknownCanonFiTag)", "(UnknownCanonFiTag)", N_("Unknown Canon File Info tag"), canonFiIfdId, makerTags, invalidTypeId, printValue)
+    };
+
+    const TagInfo* CanonMakerNote::tagListFi()
+    {
+        return tagInfoFi_;
+    }
+
+    std::ostream& CanonMakerNote::printFiFileNumber(std::ostream& os,
+                                                    const Value& value,
+                                                    const ExifData* metadata)
+    {
+        if (!metadata || value.typeId() != unsignedLong) return os << "(" << value << ")";
+        ExifData::const_iterator pos = metadata->findKey(ExifKey("Exif.Image.Model"));
+        if (pos == metadata->end()) return os << "(" << value << ")";
+
+        // Ported from Exiftool
+        std::string model = pos->toString();
+        if (   model.find("20D") != std::string::npos
+            || model.find("350D") != std::string::npos
+            || model.substr(model.size() - 8, 8) == "REBEL XT"
+            || model.find("Kiss Digital N") != std::string::npos) {
+            uint32_t val = value.toLong();
+            uint32_t dn = (val & 0xffc0) >> 6;
+            uint32_t fn = ((val >> 16) & 0xff) + ((val & 0x3f) << 8);
+            return os << std::dec << dn << "-" << std::setw(4) << std::setfill('0') << fn;
+        }
+        if (   model.find("30D") != std::string::npos
+            || model.find("400D") != std::string::npos
+            || model.find("REBEL XTi") != std::string::npos
+            || model.find("Kiss Digital X") != std::string::npos
+            || model.find("K236") != std::string::npos) {
+            uint32_t val = value.toLong();
+            uint32_t dn = (val & 0xffc00) >> 10;
+            while (dn < 100) dn += 0x40;
+            uint32_t fn = ((val & 0x3ff) << 4) + ((val >> 20) & 0x0f);
+            return os << std::dec << dn << "-" << std::setw(4) << std::setfill('0') << fn;
+        }
+
+        return os << "(" << value << ")";
+    }
+
+    std::ostream& CanonMakerNote::printFocalLength(std::ostream& os,
+                                                   const Value& value,
+                                                   const ExifData* metadata)
+    {
+        if (   !metadata
+            || value.count() < 4
+            || value.typeId() != unsignedShort) {
+            return os << value;
+        }
+
+        ExifKey key("Exif.CanonCs.Lens");
+        ExifData::const_iterator pos = metadata->findKey(key);
+        if (   pos != metadata->end()
+            && pos->value().count() >= 3
+            && pos->value().typeId() == unsignedShort) {
+            float fu = pos->value().toFloat(2);
+            if (fu != 0.0) {
+                float fl = value.toFloat(1) / fu;
+                std::ostringstream oss;
+                oss.copyfmt(os);
+                os << std::fixed << std::setprecision(1);
+                os << fl << " mm";
+                os.copyfmt(oss);
+                return os;
+            }
+        }
+
+        return os << value;
+    }
+
     std::ostream& CanonMakerNote::print0x0008(std::ostream& os,
                                               const Value& value,
                                               const ExifData*)
@@ -913,12 +1058,13 @@ namespace Exiv2 {
         return os;
     }
 
+    //! Helper structure
     struct LensTypeAndFocalLength {
-        long        lensType_;
-        std::string focalLength_;
-
+        long        lensType_;                  //!< Lens type
+        std::string focalLength_;               //!< Focal length
     };
 
+    //! Compare tag details with a lens entry
     bool operator==(const TagDetails& td, const LensTypeAndFocalLength& ltfl) {
         return (   td.val_ == ltfl.lensType_
                 && std::string(td.label_).find(ltfl.focalLength_) != std::string::npos);
