@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      convert.cpp
-  Version:   $Rev: 1681 $
+  Version:   $Rev: 1977 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   17-Mar-08, ahu: created basic converter framework
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: convert.cpp 1681 2008-11-23 13:23:53Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: convert.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -36,6 +36,7 @@ EXIV2_RCSID("@(#) $Id: convert.cpp 1681 2008-11-23 13:23:53Z ahuggel $")
 #include "exif.hpp"
 #include "iptc.hpp"
 #include "xmp.hpp"
+#include "futils.hpp"
 #include "convert.hpp"
 
 // + standard includes
@@ -46,6 +47,12 @@ EXIV2_RCSID("@(#) $Id: convert.cpp 1681 2008-11-23 13:23:53Z ahuggel $")
 #ifdef _MSC_VER
 # define snprintf _snprintf
 #endif
+#include <cstring>
+
+#ifdef EXV_HAVE_ICONV
+# include <iconv.h>
+# include <errno.h>
+#endif
 
 // Adobe XMP Toolkit
 #ifdef EXV_HAVE_XMP_TOOLKIT
@@ -66,6 +73,12 @@ namespace {
       The return code indicates if the operation was successful.
      */
     bool getTextValue(std::string& value, const Exiv2::XmpData::iterator& pos);
+
+    /*!
+      @brief Convert string charset with iconv.
+     */
+    bool convertStringCharset(std::string &str, const char* from, const char* to);
+
 }
 
 // *****************************************************************************
@@ -97,7 +110,7 @@ namespace Exiv2 {
         //! Constructor for Exif tags and XMP properties.
         Converter(ExifData& exifData, XmpData& xmpData);
         //! Constructor for Iptc tags and XMP properties.
-        Converter(IptcData& iptcData, XmpData& xmpData);
+        Converter(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset = 0);
         //@}
 
         //! @name Manipulators
@@ -277,6 +290,7 @@ namespace Exiv2 {
         ExifData *exifData_;
         IptcData *iptcData_;
         XmpData  *xmpData_;
+        const char *iptcCharset_;
 
     }; // class Converter
 
@@ -288,7 +302,7 @@ namespace Exiv2 {
         { mdExif, "Exif.Image.Compression",               "Xmp.tiff.Compression",               &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.PhotometricInterpretation", "Xmp.tiff.PhotometricInterpretation", &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Orientation",               "Xmp.tiff.Orientation",               &Converter::cnvExifValue, &Converter::cnvXmpValue },
-        { mdExif, "Exif.Image.SamplesPerPixel",           "Xmp.tiff.SamplesPerPixe",            &Converter::cnvExifValue, &Converter::cnvXmpValue },
+        { mdExif, "Exif.Image.SamplesPerPixel",           "Xmp.tiff.SamplesPerPixel",           &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.PlanarConfiguration",       "Xmp.tiff.PlanarConfiguration",       &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.YCbCrSubSampling",          "Xmp.tiff.YCbCrSubSampling",          &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.YCbCrPositioning",          "Xmp.tiff.YCbCrPositioning",          &Converter::cnvExifValue, &Converter::cnvXmpValue },
@@ -391,7 +405,7 @@ namespace Exiv2 {
         { mdIptc, "Iptc.Application2.Category",           "Xmp.photoshop.Category",             &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.SuppCategory",       "Xmp.photoshop.SupplementalCategory", &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.Keywords",           "Xmp.dc.subject",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
-        { mdIptc, "Iptc.Application2.LocationName",       "Xmp.iptc.Location",                  &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
+        { mdIptc, "Iptc.Application2.SubLocation",        "Xmp.iptc.Location",                  &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.SpecialInstructions","Xmp.photoshop.Instruction",          &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.DateCreated",        "Xmp.photoshop.DateCreated",          &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.Byline",             "Xmp.dc.creator",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
@@ -411,12 +425,12 @@ namespace Exiv2 {
     };
 
     Converter::Converter(ExifData& exifData, XmpData& xmpData)
-        : erase_(false), overwrite_(true), exifData_(&exifData), iptcData_(0), xmpData_(&xmpData)
+        : erase_(false), overwrite_(true), exifData_(&exifData), iptcData_(0), xmpData_(&xmpData), iptcCharset_(0)
     {
     }
 
-    Converter::Converter(IptcData& iptcData, XmpData& xmpData)
-        : erase_(false), overwrite_(true), exifData_(0), iptcData_(&iptcData), xmpData_(&xmpData)
+    Converter::Converter(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
+        : erase_(false), overwrite_(true), exifData_(0), iptcData_(&iptcData), xmpData_(&xmpData), iptcCharset_(iptcCharset)
     {
     }
 
@@ -590,6 +604,7 @@ namespace Exiv2 {
 
             snprintf(buf, sizeof(buf), "%.9f", dsec);
             buf[sizeof(buf) - 1] = 0;
+            buf[1] = '.'; // some locales use ','
             subsec = buf + 1;
 
             Exiv2::ExifData::iterator datePos = exifData_->findKey(ExifKey("Exif.GPSInfo.GPSDateStamp"));
@@ -627,10 +642,15 @@ namespace Exiv2 {
         }
 
         if (subsecTag) {
-            Exiv2::ExifData::iterator subsec_pos = exifData_->findKey(ExifKey(subsecTag));
+            ExifData::iterator subsec_pos = exifData_->findKey(ExifKey(subsecTag));
             if (   subsec_pos != exifData_->end()
-                && !subsec_pos->toString().empty()) {
-                subsec = std::string(".") + subsec_pos->toString();
+                && subsec_pos->typeId() == asciiString) {
+                std::string ss = subsec_pos->toString();
+                if (!ss.empty()) {
+                    bool ok = false;
+                    stringTo<long>(ss, ok);
+                    if (ok) subsec = std::string(".") + ss;
+                }
             }
             if (erase_) exifData_->erase(subsec_pos);
         }
@@ -809,7 +829,15 @@ namespace Exiv2 {
             return;
         }
         XMP_DateTime datetime;
-        SXMPUtils::ConvertToDate(value, &datetime);
+        try {
+            SXMPUtils::ConvertToDate(value, &datetime);
+        }
+        catch (const XMP_Error& e) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: Failed to convert " << from << " to " << to << " (" << e.GetErrMsg() << ")\n";
+#endif
+            return;
+        }
         char buf[30];
         if (std::string(to) != "Exif.GPSInfo.GPSTimeStamp") {
 
@@ -998,9 +1026,19 @@ namespace Exiv2 {
 #endif
             return;
         }
+        if (value.empty()) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: " << from << " is empty\n";
+#endif
+            return;
+        }
 
-        double deg, min, sec;
-        char ref, sep1, sep2;
+        double deg = 0.0;
+        double min = 0.0;
+        double sec = 0.0;
+        char ref  = '\0';
+        char sep1 = '\0';
+        char sep2 = '\0';
 
         ref = value[value.length() - 1];
         value.erase(value.length() - 1);
@@ -1056,6 +1094,7 @@ namespace Exiv2 {
                     ++pos;
                     continue;
                 }
+                if (iptcCharset_) convertStringCharset(value, iptcCharset_, "UTF-8");
                 (*xmpData_)[to] = value;
                 if (erase_) {
                     pos = iptcData_->erase(pos);
@@ -1080,13 +1119,14 @@ namespace Exiv2 {
 #endif
                 return;
             }
-            // Todo: Escape non-ASCII characters in XMP text values
             (*iptcData_)[to] = value;
+            (*iptcData_)["Iptc.Envelope.CharacterSet"] = "\033%G"; // indicate UTF-8 encoding
             if (erase_) xmpData_->erase(pos);
             return;
         }
 
         int count = pos->count();
+        bool added = false;
         for (int i = 0; i < count; ++i) {
             std::string value = pos->toString(i);
             if (!pos->value().ok()) {
@@ -1095,12 +1135,13 @@ namespace Exiv2 {
 #endif
                 continue;
             }
-            // Todo: Escape non-ASCII characters in XMP text values
             IptcKey key(to);
             Iptcdatum id(key);
             id.setValue(value);
             iptcData_->add(id);
+            added = true;
         }
+        if (added) (*iptcData_)["Iptc.Envelope.CharacterSet"] = "\033%G"; // indicate UTF-8 encoding
         if (erase_) xmpData_->erase(pos);
     }
 
@@ -1208,6 +1249,7 @@ namespace Exiv2 {
 #endif
     }
 
+
     // *************************************************************************
     // free functions
     void copyExifToXmp(const ExifData& exifData, XmpData& xmpData)
@@ -1242,15 +1284,20 @@ namespace Exiv2 {
         converter.syncExifWithXmp();
     }
 
-    void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData)
+    void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
     {
-        Converter converter(const_cast<IptcData&>(iptcData), xmpData);
+        if (!iptcCharset) iptcCharset = iptcData.detectCharset();
+        if (!iptcCharset) iptcCharset = "ISO-8859-1";
+
+        Converter converter(const_cast<IptcData&>(iptcData), xmpData, iptcCharset);
         converter.cnvToXmp();
     }
 
-    void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData)
+    void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
     {
-        Converter converter(const_cast<IptcData&>(iptcData), xmpData);
+        if (!iptcCharset) iptcCharset = iptcData.detectCharset();
+        if (!iptcCharset) iptcCharset = "ISO-8859-1";
+        Converter converter(const_cast<IptcData&>(iptcData), xmpData, iptcCharset);
         converter.setErase();
         converter.cnvToXmp();
     }
@@ -1301,4 +1348,53 @@ namespace {
         return pos->value().ok();
     }
 
+    bool convertStringCharset(std::string &str, const char* from, const char* to)
+    {
+        if (0 == strcmp(from, to)) return true; // nothing to do
+#if defined EXV_HAVE_ICONV
+        bool ret = true;
+        iconv_t cd;
+        cd = iconv_open(to, from);
+        if (cd == (iconv_t)(-1)) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: iconv_open: " << Exiv2::strError() << "\n";
+#endif
+            return false;
+        }
+        std::string outstr;
+        EXV_ICONV_CONST char *inptr = const_cast<char *>(str.c_str());
+        size_t inbytesleft = str.length();
+
+        while (inbytesleft) {
+            char outbuf[100];
+            char *outptr = outbuf;
+            size_t outbytesleft = sizeof(outbuf) - 1;
+            size_t rc = iconv(cd,
+                              &inptr,
+                              &inbytesleft,
+                              &outptr,
+                              &outbytesleft);
+            if (rc == size_t(-1) && errno != E2BIG) {
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << "Warning: iconv: "
+                          << Exiv2::strError()
+                          << " inbytesleft = " << inbytesleft << "\n";
+#endif
+                ret = false;
+                break;
+            }
+            *outptr = '\0';
+            outstr.append(outbuf);
+        }
+        if (cd != (iconv_t)(-1)) {
+            iconv_close(cd);
+        }
+
+        if (ret) str = outstr;
+        return ret;
+#else // !EXV_HAVE_ICONV
+        return false;
+#endif // EXV_HAVE_ICONV
+    }
+
 }
