@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    convert.hpp
   @brief   Exif and IPTC conversions to and from XMP
-  @version $Rev: 1571 $
+  @version $Rev: 1937 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a><BR>
            Vladimir Nadvornik (vn)
@@ -69,9 +69,9 @@ namespace Exiv2 {
     EXIV2API void syncExifWithXmp(ExifData& exifData, XmpData& xmpData);
 
     //! Convert (copy) IPTC datasets to XMP properties.
-    EXIV2API void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData);
+    EXIV2API void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData, const char *iptcCharset = 0);
     //! Convert (move) IPTC datasets to XMP properties, remove converted IPTC datasets.
-    EXIV2API void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData);
+    EXIV2API void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset = 0);
 
     //! Convert (copy) XMP properties to IPTC datasets.
     EXIV2API void copyXmpToIptc(const XmpData& xmpData, IptcData& iptcData);
