@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      cr2image.cpp
-  Version:   $Rev: 1594 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   22-Apr-06, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: cr2image.cpp 1594 2008-09-18 11:03:20Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: cr2image.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -41,7 +41,6 @@ EXIV2_RCSID("@(#) $Id: cr2image.cpp 1594 2008-09-18 11:03:20Z ahuggel $")
 #include "cr2image_int.hpp"
 #include "tiffcomposite_int.hpp"
 #include "tiffimage_int.hpp"
-#include "tiffvisitor_int.hpp"
 #include "image.hpp"
 #include "error.hpp"
 #include "futils.hpp"
@@ -64,6 +63,11 @@ namespace Exiv2 {
     {
     } // Cr2Image::Cr2Image
 
+    std::string Cr2Image::mimeType() const
+    {
+        return "image/x-canon-cr2";
+    }
+
     int Cr2Image::pixelWidth() const
     {
         ExifData::const_iterator imageWidth = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
@@ -143,7 +147,7 @@ namespace Exiv2 {
                                         xmpData,
                                         pData,
                                         size,
-                                        TiffCreator::create,
+                                        Tag::root,
                                         TiffMapping::findDecoder,
                                         &cr2Header);
     }
@@ -237,10 +241,10 @@ namespace Exiv2 {
         return true;
     } // Cr2Header::read
 
-    uint32_t Cr2Header::write(Blob& blob) const
+    DataBuf Cr2Header::write() const
     {
         // Todo: Implement me!
-        return 0;
+        return DataBuf();
     }
 
 }}                                      // namespace Internal, Exiv2
