@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      crwimage.cpp
-  Version:   $Rev: 1704 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   28-Aug-05, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: crwimage.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: crwimage.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // Define DEBUG to output debug information to std::cerr, e.g, by calling make
 // like this: make DEFS=-DDEBUG crwimage.o
@@ -95,6 +95,11 @@ namespace Exiv2 {
     {
     } // CrwImage::CrwImage
 
+    std::string CrwImage::mimeType() const
+    {
+        return "image/x-canon-crw";
+    }
+
     int CrwImage::pixelWidth() const
     {
         Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
@@ -338,6 +343,10 @@ namespace Exiv2 {
         if (isAllocated_) delete[] pData_;
     }
 
+    CiffEntry::~CiffEntry()
+    {
+    }
+
     CiffDirectory::~CiffDirectory()
     {
         Components::iterator b = components_.begin();
