@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      datasets.cpp
-  Version:   $Rev: 1614 $
+  Version:   $Rev: 1937 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
              Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   24-Jul-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: datasets.cpp 1614 2008-09-27 03:11:05Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: datasets.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -141,7 +141,7 @@ namespace Exiv2 {
                 "invocation or designation of coded character sets. The control functions follow "
                 "the ISO 2022 standard and may consist of the escape control "
                 "character and one or more graphic characters."),
-                false, false, 0, 32, Exiv2::undefined, IptcDataSets::envelope, ""),
+                false, false, 0, 32, Exiv2::string, IptcDataSets::envelope, ""),
         DataSet(IptcDataSets::UNO, "UNO", N_("Unique Name Object"),
                 N_("This tag provide a globally unique "
                 "identification for objects as specified in the IIM, independent of "
@@ -622,6 +622,46 @@ namespace Exiv2 {
         return *this;
     }
 
+    std::string IptcKey::key() const
+    {
+        return key_;
+    }
+
+    const char* IptcKey::familyName() const
+    {
+        return familyName_;
+    }
+
+    std::string IptcKey::groupName() const
+    {
+        return recordName();
+    }
+
+    std::string IptcKey::tagName() const
+    {
+        return IptcDataSets::dataSetName(tag_, record_);
+    }
+
+    std::string IptcKey::tagLabel() const
+    {
+        return IptcDataSets::dataSetTitle(tag_, record_);
+    }
+
+    uint16_t IptcKey::tag() const
+    {
+        return tag_;
+    }
+
+    std::string IptcKey::recordName() const
+    {
+        return IptcDataSets::recordName(record_);
+    }
+
+    uint16_t IptcKey::record() const
+    {
+        return record_;
+    }
+
     IptcKey::AutoPtr IptcKey::clone() const
     {
         return AutoPtr(clone_());
