@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      error.cpp
-  Version:   $Rev: 1677 $
+  Version:   $Rev: 1957 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   02-Apr-05, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: error.cpp 1677 2008-11-16 05:25:04Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: error.cpp 1957 2009-12-13 10:27:43Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -49,7 +49,7 @@ namespace {
         { -1, N_("Error %0: arg1=%1, arg2=%2, arg3=%3.") },
         {  0, N_("Success") },
         {  1, "%1" }, // %1=error message
-        {  2, "%1: %2 (%3)" }, // %1=path, %2=strerror, %3=function that failed
+        {  2, "%1: Call to `%3' failed: %2" }, // %1=path, %2=strerror, %3=function that failed
         {  3, N_("This does not look like a %1 image") }, // %1=Image type
         {  4, N_("Invalid dataset name `%1'") }, // %1=dataset name
         {  5, N_("Invalid record name `%1'") }, // %1=record name
@@ -78,7 +78,7 @@ namespace {
         { 28, N_("Invalid charset: `%1'") }, // %1=charset name
         { 29, N_("Unsupported date format") },
         { 30, N_("Unsupported time format") },
-        { 31, N_("Writing to %2 images is not supported") }, // %2=image format
+        { 31, N_("Writing to %1 images is not supported") }, // %1=image format
         { 32, N_("Setting %1 in %2 images is not supported") }, // %1=metadata type, %2=image format
         { 33, N_("This does not look like a CRW image") },
         { 34, N_("%1: Not supported") }, // %1=function
@@ -109,6 +109,30 @@ namespace {
 // class member definitions
 namespace Exiv2 {
 
+    AnyError::~AnyError() throw()
+    {
+    }
+
+    Error::Error(int code)
+        : code_(code), count_(0)
+    {
+        setMsg();
+    }
+
+    Error::~Error() throw()
+    {
+    }
+
+    int Error::code() const throw()
+    {
+        return code_;
+    }
+
+    const char* Error::what() const throw()
+    {
+        return msg_.c_str();
+    }
+
     int Error::errorIdx(int code)
     {
         int idx;
