@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      exif.cpp
-  Version:   $Rev: 1704 $
+  Version:   $Rev: 1977 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: exif.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: exif.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -45,6 +45,8 @@ EXIV2_RCSID("@(#) $Id: exif.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
 #include "error.hpp"
 #include "basicio.hpp"
 #include "tiffimage.hpp"
+#include "tiffimage_int.hpp"
+#include "tiffcomposite_int.hpp" // for Tag::root
 
 // + standard includes
 #include <iostream>
@@ -57,18 +59,25 @@ EXIV2_RCSID("@(#) $Id: exif.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
 
 // *****************************************************************************
 namespace {
-    //! Unary predicate that matches an Exifdatum with a given IfdId.
-    class FindExifdatum {
+
+    //! Unary predicate that matches a Exifdatum with a given key
+    class FindExifdatumByKey {
     public:
-        //! Constructor, initializes the object with the IfdId to look for.
-        FindExifdatum(Exiv2::IfdId ifdId) : ifdId_(ifdId) {}
-        //! Returns true if IFD id matches.
-        bool operator()(const Exiv2::Exifdatum& md) const { return ifdId_ == md.ifdId(); }
+        //! Constructor, initializes the object with the key to look for
+        FindExifdatumByKey(const std::string& key) : key_(key) {}
+        /*!
+          @brief Returns true if the key of \em exifdatum is equal
+                 to that of the object.
+        */
+        bool operator()(const Exiv2::Exifdatum& exifdatum) const
+        {
+            return key_ == exifdatum.key();
+        }
 
     private:
-        Exiv2::IfdId ifdId_;
+        std::string key_;
 
-    }; // class FindExifdatum
+    }; // class FindExifdatumByKey
 
     /*!
       @brief Exif %Thumbnail image. This abstract base class provides the
@@ -108,6 +117,13 @@ namespace {
                  (".tif", ".jpg").
          */
         virtual const char* extension() const =0;
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like extension() but returns the extension in a wchar_t.
+          @note This function is only available on Windows.
+         */
+        virtual const wchar_t* wextension() const =0;
+#endif
         //@}
 
     }; // class Thumbnail
@@ -129,6 +145,9 @@ namespace {
         Exiv2::DataBuf copy(const Exiv2::ExifData& exifData) const;
         const char* mimeType() const;
         const char* extension() const;
+#ifdef EXV_UNICODE_PATH
+        const wchar_t* wextension() const;
+#endif
         //@}
 
     }; // class TiffThumbnail
@@ -150,6 +169,9 @@ namespace {
         Exiv2::DataBuf copy(const Exiv2::ExifData& exifData) const;
         const char* mimeType() const;
         const char* extension() const;
+#ifdef EXV_UNICODE_PATH
+        const wchar_t* wextension() const;
+#endif
         //@}
 
     }; // class JpegThumbnail
@@ -166,6 +188,8 @@ namespace {
 // class member definitions
 namespace Exiv2 {
 
+    using namespace Internal;
+
     /*!
       @brief Set the value of \em exifDatum to \em value. If the object already
              has a value, it is replaced. Otherwise a new ValueType\<T\> value
@@ -283,6 +307,131 @@ namespace Exiv2 {
         return value_->read(value);
     }
 
+    int Exifdatum::setDataArea(const byte* buf, long len)
+    {
+        return value_.get() == 0 ? -1 : value_->setDataArea(buf, len);
+    }
+
+    std::string Exifdatum::key() const
+    {
+        return key_.get() == 0 ? "" : key_->key();
+    }
+
+    const char* Exifdatum::familyName() const
+    {
+        return key_.get() == 0 ? "" : key_->familyName();
+    }
+
+    std::string Exifdatum::groupName() const
+    {
+        return key_.get() == 0 ? "" : key_->groupName();
+    }
+
+    std::string Exifdatum::tagName() const
+    {
+        return key_.get() == 0 ? "" : key_->tagName();
+    }
+
+    std::string Exifdatum::tagLabel() const
+    {
+        return key_.get() == 0 ? "" : key_->tagLabel();
+    }
+
+    uint16_t Exifdatum::tag() const
+    {
+        return key_.get() == 0 ? 0xffff : key_->tag();
+    }
+
+    IfdId Exifdatum::ifdId() const
+    {
+        return key_.get() == 0 ? ifdIdNotSet : key_->ifdId();
+    }
+
+    const char* Exifdatum::ifdName() const
+    {
+        return key_.get() == 0 ? "" : key_->ifdName();
+    }
+
+    std::string Exifdatum::ifdItem() const
+    {
+        return key_.get() == 0 ? "" : key_->ifdItem();
+    }
+
+    int Exifdatum::idx() const
+    {
+        return key_.get() == 0 ? 0 : key_->idx();
+    }
+
+    long Exifdatum::copy(byte* buf, ByteOrder byteOrder) const
+    {
+        return value_.get() == 0 ? 0 : value_->copy(buf, byteOrder);
+    }
+
+    TypeId Exifdatum::typeId() const
+    {
+        return value_.get() == 0 ? invalidTypeId : value_->typeId();
+    }
+
+    const char* Exifdatum::typeName() const
+    {
+        return TypeInfo::typeName(typeId());
+    }
+
+    long Exifdatum::typeSize() const
+    {
+        return TypeInfo::typeSize(typeId());
+    }
+
+    long Exifdatum::count() const
+    {
+        return value_.get() == 0 ? 0 : value_->count();
+    }
+
+    long Exifdatum::size() const
+    {
+        return value_.get() == 0 ? 0 : value_->size();
+    }
+
+    std::string Exifdatum::toString() const
+    {
+        return value_.get() == 0 ? "" : value_->toString();
+    }
+
+    std::string Exifdatum::toString(long n) const
+    {
+        return value_.get() == 0 ? "" : value_->toString(n);
+    }
+
+    long Exifdatum::toLong(long n) const
+    {
+        return value_.get() == 0 ? -1 : value_->toLong(n);
+    }
+
+    float Exifdatum::toFloat(long n) const
+    {
+        return value_.get() == 0 ? -1 : value_->toFloat(n);
+    }
+
+    Rational Exifdatum::toRational(long n) const
+    {
+        return value_.get() == 0 ? Rational(-1, 1) : value_->toRational(n);
+    }
+
+    Value::AutoPtr Exifdatum::getValue() const
+    {
+        return value_.get() == 0 ? Value::AutoPtr(0) : value_->clone();
+    }
+
+    long Exifdatum::sizeDataArea() const
+    {
+        return value_.get() == 0 ? 0 : value_->sizeDataArea();
+    }
+
+    DataBuf Exifdatum::dataArea() const
+    {
+        return value_.get() == 0 ? DataBuf(0, 0) : value_->dataArea();
+    }
+
     ExifThumbC::ExifThumbC(const ExifData& exifData)
         : exifData_(exifData)
     {
@@ -305,6 +454,18 @@ namespace Exiv2 {
         return Exiv2::writeFile(buf, name);
     }
 
+#ifdef EXV_UNICODE_PATH
+    long ExifThumbC::writeFile(const std::wstring& wpath) const
+    {
+        Thumbnail::AutoPtr thumbnail = Thumbnail::create(exifData_);
+        if (thumbnail.get() == 0) return 0;
+        std::wstring name = wpath + thumbnail->wextension();
+        DataBuf buf(thumbnail->copy(exifData_));
+        if (buf.size_ == 0) return 0;
+        return Exiv2::writeFile(buf, name);
+    }
+
+#endif
     const char* ExifThumbC::mimeType() const
     {
         Thumbnail::AutoPtr thumbnail = Thumbnail::create(exifData_);
@@ -319,6 +480,15 @@ namespace Exiv2 {
         return thumbnail->extension();
     }
 
+#ifdef EXV_UNICODE_PATH
+    const wchar_t* ExifThumbC::wextension() const
+    {
+        Thumbnail::AutoPtr thumbnail = Thumbnail::create(exifData_);
+        if (thumbnail.get() == 0) return EXV_WIDEN("");
+        return thumbnail->wextension();
+    }
+
+#endif
     ExifThumb::ExifThumb(ExifData& exifData)
         : ExifThumbC(exifData), exifData_(exifData)
     {
@@ -335,6 +505,19 @@ namespace Exiv2 {
         setJpegThumbnail(thumb.pData_, thumb.size_, xres, yres, unit);
     }
 
+#ifdef EXV_UNICODE_PATH
+    void ExifThumb::setJpegThumbnail(
+        const std::wstring& wpath,
+              URational     xres,
+              URational     yres,
+              uint16_t      unit
+    )
+    {
+        DataBuf thumb = readFile(wpath); // may throw
+        setJpegThumbnail(thumb.pData_, thumb.size_, xres, yres, unit);
+    }
+
+#endif
     void ExifThumb::setJpegThumbnail(
         const byte*     buf,
               long      size,
@@ -355,6 +538,14 @@ namespace Exiv2 {
         setJpegThumbnail(thumb.pData_, thumb.size_);
     }
 
+#ifdef EXV_UNICODE_PATH
+    void ExifThumb::setJpegThumbnail(const std::wstring& wpath)
+    {
+        DataBuf thumb = readFile(wpath); // may throw
+        setJpegThumbnail(thumb.pData_, thumb.size_);
+    }
+
+#endif
     void ExifThumb::setJpegThumbnail(const byte* buf, long size)
     {
         exifData_["Exif.Thumbnail.Compression"] = uint16_t(6);
@@ -394,13 +585,13 @@ namespace Exiv2 {
     ExifData::const_iterator ExifData::findKey(const ExifKey& key) const
     {
         return std::find_if(exifMetadata_.begin(), exifMetadata_.end(),
-                            FindMetadatumByKey(key.key()));
+                            FindExifdatumByKey(key.key()));
     }
 
     ExifData::iterator ExifData::findKey(const ExifKey& key)
     {
         return std::find_if(exifMetadata_.begin(), exifMetadata_.end(),
-                            FindMetadatumByKey(key.key()));
+                            FindExifdatumByKey(key.key()));
     }
 
     void ExifData::clear()
@@ -497,7 +688,9 @@ namespace Exiv2 {
             subImage2Id,
             subImage3Id,
             subImage4Id,
-            ifd2Id
+            panaRawIfdId,
+            ifd2Id,
+            ifd3Id
         };
         for (unsigned int i = 0; i < EXV_COUNTOF(filteredIfds); ++i) {
 #ifdef DEBUG
@@ -507,21 +700,27 @@ namespace Exiv2 {
         }
 
         // IPTC and XMP are stored elsewhere, not in the Exif APP1 segment.
-        const IptcData iptcData;
-        const XmpData  xmpData;
+        const IptcData emptyIptc;
+        const XmpData  emptyXmp;
 
-        // Encode and check if the results fits into a JPEG Exif APP1 segment
-        WriteMethod wm = TiffParser::encode(blob,
-                                            pData,
-                                            size,
-                                            byteOrder,
-                                            ed,
-                                            iptcData,
-                                            xmpData);
-        if (blob.size() <= 65527) return wm;
+        // Encode and check if the result fits into a JPEG Exif APP1 segment
+        MemIo mio1;
+        std::auto_ptr<TiffHeaderBase> header(new TiffHeader(byteOrder));
+        WriteMethod wm = TiffParserWorker::encode(mio1,
+                                                  pData,
+                                                  size,
+                                                  ed,
+                                                  emptyIptc,
+                                                  emptyXmp,
+                                                  Tag::root,
+                                                  TiffMapping::findEncoder,
+                                                  header.get());
+        if (mio1.size() <= 65527) {
+            append(blob, mio1.mmap(), mio1.size());
+            return wm;
+        }
 
         // If it doesn't fit, remove additional tags
-        blob.clear();
 
         // Delete preview tags if the preview is larger than 32kB.
         // Todo: Enhance preview classes to be able to write and delete previews and use that instead.
@@ -603,16 +802,20 @@ namespace Exiv2 {
         }
 
         // Encode the remaining Exif tags again, don't care if it fits this time
-        wm = TiffParser::encode(blob,
-                                pData,
-                                size,
-                                byteOrder,
-                                ed,
-                                iptcData,
-                                xmpData);
+        MemIo mio2;
+        wm = TiffParserWorker::encode(mio2,
+                                      pData,
+                                      size,
+                                      ed,
+                                      emptyIptc,
+                                      emptyXmp,
+                                      Tag::root,
+                                      TiffMapping::findEncoder,
+                                      header.get());
+        append(blob, mio2.mmap(), mio2.size());
 #ifdef DEBUG
         if (wm == wmIntrusive) {
-            std::cerr << "SIZE OF EXIF DATA IS " << std::dec << blob.size() << " BYTES\n";
+            std::cerr << "SIZE OF EXIF DATA IS " << std::dec << io.size() << " BYTES\n";
         }
         else {
             std::cerr << "SIZE DOESN'T MATTER, NON-INTRUSIVE WRITING USED\n";
@@ -664,6 +867,13 @@ namespace {
         return ".tif";
     }
 
+#ifdef EXV_UNICODE_PATH
+    const wchar_t* TiffThumbnail::wextension() const
+    {
+        return EXV_WIDEN(".tif");
+    }
+
+#endif
     Exiv2::DataBuf TiffThumbnail::copy(const Exiv2::ExifData& exifData) const
     {
         Exiv2::ExifData thumb;
@@ -675,11 +885,11 @@ namespace {
             }
         }
 
-        Exiv2::Blob blob;
+        Exiv2::MemIo io;
         const Exiv2::IptcData emptyIptc;
         const Exiv2::XmpData  emptyXmp;
-        Exiv2::TiffParser::encode(blob, 0, 0, Exiv2::littleEndian, thumb, emptyIptc, emptyXmp);
-        return Exiv2::DataBuf((blob.size() > 0 ? &blob[0] : 0), static_cast<long>(blob.size()));
+        Exiv2::TiffParser::encode(io, 0, 0, Exiv2::littleEndian, thumb, emptyIptc, emptyXmp);
+        return io.read(io.size());
     }
 
     const char* JpegThumbnail::mimeType() const
@@ -692,6 +902,13 @@ namespace {
         return ".jpg";
     }
 
+#ifdef EXV_UNICODE_PATH
+    const wchar_t* JpegThumbnail::wextension() const
+    {
+        return EXV_WIDEN(".jpg");
+    }
+
+#endif
     Exiv2::DataBuf JpegThumbnail::copy(const Exiv2::ExifData& exifData) const
     {
         Exiv2::ExifKey key("Exif.Thumbnail.JPEGInterchangeFormat");
@@ -711,7 +928,10 @@ namespace {
 
     void eraseIfd(Exiv2::ExifData& ed, Exiv2::IfdId ifdId)
     {
-        ed.erase(std::remove_if(ed.begin(), ed.end(), FindExifdatum(ifdId)), ed.end());
+        ed.erase(std::remove_if(ed.begin(),
+                                ed.end(),
+                                Exiv2::Internal::FindExifdatum(ifdId)),
+                 ed.end());
     }
     //! @endcond
 }
