@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    exif.hpp
   @brief   Encoding and decoding of Exif data
-  @version $Rev: 1692 $
+  @version $Rev: 1937 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created
@@ -149,37 +149,26 @@ namespace Exiv2 {
           @return Return -1 if the %Exifdatum does not have a value yet or the
                   value has no data area, else 0.
          */
-        int setDataArea(const byte* buf, long len)
-            { return value_.get() == 0 ? -1 : value_->setDataArea(buf, len); }
+        int setDataArea(const byte* buf, long len);
         //@}
 
         //! @name Accessors
         //@{
         //! Return the key of the %Exifdatum.
-        std::string key() const
-            { return key_.get() == 0 ? "" : key_->key(); }
-        const char* familyName() const
-            { return key_.get() == 0 ? "" : key_->familyName(); }
-        std::string groupName() const
-            { return key_.get() == 0 ? "" : key_->groupName(); }
-        std::string tagName() const
-            { return key_.get() == 0 ? "" : key_->tagName(); }
-        std::string tagLabel() const
-            { return key_.get() == 0 ? "" : key_->tagLabel(); }
-        uint16_t tag() const
-            { return key_.get() == 0 ? 0xffff : key_->tag(); }
+        std::string key() const;
+        const char* familyName() const;
+        std::string groupName() const;
+        std::string tagName() const;
+        std::string tagLabel() const;
+        uint16_t tag() const;
         //! Return the IFD id
-        IfdId ifdId() const
-            { return key_.get() == 0 ? ifdIdNotSet : key_->ifdId(); }
+        IfdId ifdId() const;
         //! Return the name of the IFD
-        const char* ifdName() const
-            { return key_.get() == 0 ? "" : key_->ifdName(); }
+        const char* ifdName() const;
         //! Return the related image item (deprecated)
-        std::string ifdItem() const
-            { return key_.get() == 0 ? "" : key_->ifdItem(); }
+        std::string ifdItem() const;
         //! Return the index (unique id of this key within the original IFD)
-        int idx() const
-            { return key_.get() == 0 ? 0 : key_->idx(); }
+        int idx() const;
         /*!
           @brief Write value to a data buffer and return the number
                  of bytes written.
@@ -191,41 +180,28 @@ namespace Exiv2 {
           @param byteOrder Applicable byte order (little or big endian).
           @return Number of characters written.
         */
-        long copy(byte* buf, ByteOrder byteOrder) const
-            { return value_.get() == 0 ? 0 : value_->copy(buf, byteOrder); }
+        long copy(byte* buf, ByteOrder byteOrder) const;
         std::ostream& write(std::ostream& os, const ExifData* pMetadata =0) const;
         //! Return the type id of the value
-        TypeId typeId() const
-            { return value_.get() == 0 ? invalidTypeId : value_->typeId(); }
+        TypeId typeId() const;
         //! Return the name of the type
-        const char* typeName() const
-            { return TypeInfo::typeName(typeId()); }
+        const char* typeName() const;
         //! Return the size in bytes of one component of this type
-        long typeSize() const
-            { return TypeInfo::typeSize(typeId()); }
+        long typeSize() const;
         //! Return the number of components in the value
-        long count() const
-            { return value_.get() == 0 ? 0 : value_->count(); }
+        long count() const;
         //! Return the size of the value in bytes
-        long size() const
-            { return value_.get() == 0 ? 0 : value_->size(); }
+        long size() const;
         //! Return the value as a string.
-        std::string toString() const
-            { return value_.get() == 0 ? "" : value_->toString(); }
-        std::string toString(long n) const
-            { return value_.get() == 0 ? "" : value_->toString(n); }
-        long toLong(long n =0) const
-            { return value_.get() == 0 ? -1 : value_->toLong(n); }
-        float toFloat(long n =0) const
-            { return value_.get() == 0 ? -1 : value_->toFloat(n); }
-        Rational toRational(long n =0) const
-            { return value_.get() == 0 ? Rational(-1, 1) : value_->toRational(n); }
-        Value::AutoPtr getValue() const
-            { return value_.get() == 0 ? Value::AutoPtr(0) : value_->clone(); }
+        std::string toString() const;
+        std::string toString(long n) const;
+        long toLong(long n =0) const;
+        float toFloat(long n =0) const;
+        Rational toRational(long n =0) const;
+        Value::AutoPtr getValue() const;
         const Value& value() const;
         //! Return the size of the data area.
-        long sizeDataArea() const
-            { return value_.get() == 0 ? 0 : value_->sizeDataArea(); }
+        long sizeDataArea() const;
         /*!
           @brief Return a copy of the data area of the value. The caller owns
                  this copy and %DataBuf ensures that it will be deleted.
@@ -238,8 +214,7 @@ namespace Exiv2 {
                   %DataBuf if the value does not have a data area assigned or the
                   value is not set.
          */
-        DataBuf dataArea() const
-            { return value_.get() == 0 ? DataBuf(0, 0) : value_->dataArea(); }
+        DataBuf dataArea() const;
         //@}
 
     private:
@@ -286,6 +261,13 @@ namespace Exiv2 {
           @return The number of bytes written.
         */
         long writeFile(const std::string& path) const;
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like writeFile() but accepts a unicode path in an std::wstring.
+          @note This function is only available on Windows.
+         */
+        long writeFile(const std::wstring& wpath) const;
+#endif
         /*!
           @brief Return the MIME type of the thumbnail, either \c "image/tiff"
                  or \c "image/jpeg".
@@ -296,6 +278,13 @@ namespace Exiv2 {
                  (".tif" or ".jpg").
          */
         const char* extension() const;
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like extension() but returns the extension in a wchar_t.
+          @note This function is only available on Windows.
+         */
+        const wchar_t* wextension() const;
+#endif
         //@}
 
     private:
@@ -348,6 +337,19 @@ namespace Exiv2 {
                   URational    yres,
                   uint16_t     unit
         );
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like setJpegThumbnail() but accepts a unicode path in an
+                 std::wstring.
+          @note This function is only available on Windows.
+         */
+        void setJpegThumbnail(
+            const std::wstring& wpath,
+                  URational     xres,
+                  URational     yres,
+                  uint16_t      unit
+        );
+#endif
         /*!
           @brief Set the Exif thumbnail to the JPEG image pointed to by \em buf,
                  and size \em size. Set XResolution, YResolution and
@@ -386,6 +388,14 @@ namespace Exiv2 {
           @note  Additional existing Exif thumbnail tags are not modified.
          */
         void setJpegThumbnail(const std::string& path);
+#ifdef EXV_UNICODE_PATH
+        /*!
+          @brief Like setJpegThumbnail(const std::string& path) but accepts a
+                 unicode path in an std::wstring.
+          @note This function is only available on Windows.
+         */
+        void setJpegThumbnail(const std::wstring& wpath);
+#endif
         /*!
           @brief Set the Exif thumbnail to the JPEG image pointed to by \em buf,
                  and size \em size.
@@ -518,15 +528,24 @@ namespace Exiv2 {
 
     /*!
       @brief Stateless parser class for Exif data. Images use this class to
-             decode and encode binary Exif data. See class TiffParser for details.
+             decode and encode binary Exif data.
+
+      @note  Encode is lossy and is not the inverse of decode.
      */
     class EXIV2API ExifParser {
     public:
         /*!
           @brief Decode metadata from a buffer \em pData of length \em size
                  with binary Exif data to the provided metadata container.
-                 Return byte order in which the data is encoded.
-                 See TiffParser::decode().
+
+                 The buffer must start with a TIFF header. Return byte order
+                 in which the data is encoded.
+
+          @param exifData Exif metadata container.
+          @param pData 	  Pointer to the data buffer. Must point to data in
+                          binary Exif format; no checks are performed.
+          @param size 	  Length of the data buffer
+          @return Byte order in which the data is encoded.
         */
         static ByteOrder decode(
                   ExifData& exifData,
@@ -534,8 +553,42 @@ namespace Exiv2 {
                   uint32_t  size
         );
         /*!
-          @brief Encode metadata from the provided metadata to Exif format.
-                 See TiffParser::encode().
+          @brief Encode Exif metadata from the provided metadata to binary Exif
+                 format.
+
+          The original binary Exif data in the memory block \em pData, \em size
+          is parsed and updated in-place if possible ("non-intrusive"
+          writing). If that is not possible (e.g., if new tags were added), the
+          entire Exif structure is re-written to the \em blob ("intrusive"
+          writing). The return value indicates which write method was used. If
+          it is \c wmNonIntrusive, the original memory \em pData, \em size
+          contains the result and \em blob is empty. If the return value is
+          \c wmIntrusive, a new Exif structure was created and returned in
+          \em blob. The memory block \em pData, \em size may be partly updated in
+          this case and should not be used anymore.
+
+          Encode is a lossy operation. It attempts to fit the Exif data into a
+          binary block suitable as the payload of a JPEG APP1 Exif segment,
+          which can be at most 65527 bytes large. Encode omits IFD0 tags that
+          are "not recorded" in compressed images according to the Exif 2.2
+          specification. It also doesn't write tags in groups which do not occur
+          in JPEG images. If the resulting binary block is larger than allowed,
+          it further deletes specific large preview tags and unknown tags. The
+          operation succeeds even if the end result is still larger than the
+          allowed size. Application should therefore always check the size of
+          the \em blob.
+
+          @param blob      Container for the binary Exif data if "intrusive"
+                           writing is necessary. Empty otherwise.
+          @param pData     Pointer to the binary Exif data buffer. Must
+                           point to data in Exif format; no checks are
+                           performed. Will be modified if "non-intrusive"
+                           writing is possible.
+          @param size      Length of the data buffer.
+          @param byteOrder Byte order to use.
+          @param exifData  Exif metadata container.
+
+          @return Write method used.
         */
         static WriteMethod encode(
                   Blob&     blob,
@@ -546,10 +599,9 @@ namespace Exiv2 {
         );
         /*!
           @brief Encode metadata from the provided metadata to Exif format.
-                 See TiffParser::encode().
 
-          Encode Exif metadata from the \em ExifData container to binary format
-          in the \em blob encoded in \em byteOrder.
+          Encode Exif metadata from the \em ExifData container to binary Exif
+          format in the \em blob, encoded in \em byteOrder.
 
           This simpler encode method uses "intrusive" writing, i.e., it builds
           the binary representation of the metadata from scratch. It does not
@@ -560,6 +612,10 @@ namespace Exiv2 {
 
           This is just an inline wrapper for
           ExifParser::encode(blob, 0, 0, byteOrder, exifData).
+
+          @param blob      Container for the binary Exif data.
+          @param byteOrder Byte order to use.
+          @param exifData  Exif metadata container.
         */
         static void encode(
                   Blob&     blob,
