@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -22,13 +22,13 @@
   Abstract:  Command line program to display and manipulate image metadata.
 
   File:      exiv2.cpp
-  Version:   $Rev: 1701 $
+  Version:   $Rev: 1977 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   10-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: exiv2.cpp 1701 2008-12-11 16:54:56Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: exiv2.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -81,7 +81,7 @@ namespace {
     int parseCommonTargets(const std::string& optarg,
                            const std::string& action);
 
-    /*! 
+    /*!
       @brief Parse numbers separated by commas into container
       @param previewNumbers Container for the numbers
       @param optarg Option arguments
@@ -200,7 +200,7 @@ void Params::cleanup()
 void Params::version(std::ostream& os) const
 {
     os << EXV_PACKAGE_STRING << "\n"
-       << _("Copyright (C) 2004-2008 Andreas Huggel.\n")
+       << _("Copyright (C) 2004-2009 Andreas Huggel.\n")
        << "\n"
        << _("This program is free software; you can redistribute it and/or\n"
             "modify it under the terms of the GNU General Public License\n"
@@ -249,7 +249,7 @@ void Params::help(std::ostream& os) const
        << _("   -V      Show the program version and exit.\n")
        << _("   -v      Be verbose during the program run.\n")
        << _("   -b      Show large binary values.\n")
-       << _("   -u      Don't show unknown tags.\n")
+       << _("   -u      Show unknown tags.\n")
        << _("   -k      Preserve file timestamps (keep).\n")
        << _("   -t      Also set the file timestamp in 'rename' action (overrides -k).\n")
        << _("   -T      Only set the file timestamp in 'rename' action, do not rename\n"
@@ -275,7 +275,7 @@ void Params::help(std::ostream& os) const
        << _("             E : include Exif tags in the list\n")
        << _("             I : IPTC datasets\n")
        << _("             X : XMP properties\n")
-       << _("             x : print a column with the tag value\n")
+       << _("             x : print a column with the tag number\n")
        << _("             g : group name\n")
        << _("             k : key\n")
        << _("             l : tag label\n")
@@ -328,7 +328,7 @@ int Params::option(int opt, const std::string& optarg, int optopt)
     case 'v': verbose_ = true; break;
     case 'k': preserve_ = true; break;
     case 'b': binary_ = false; break;
-    case 'u': unknown_ = true; break;
+    case 'u': unknown_ = false; break;
     case 'f': force_ = true; fileExistsPolicy_ = overwritePolicy; break;
     case 'F': force_ = true; fileExistsPolicy_ = renamePolicy; break;
     case 'r': rc = evalRename(opt, optarg); break;
