@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -19,15 +19,15 @@
  * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
-  File:      utils.cpp
-  Version:   $Rev: 1366 $
+  File:      futils.cpp
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              02-Apr-05, ahu: moved to Exiv2 namespace
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: futils.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: futils.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -69,19 +69,31 @@ namespace Exiv2 {
     bool fileExists(const std::string& path, bool ct)
     {
         struct stat buf;
-        int ret = stat(path.c_str(), &buf);
+		int ret = ::stat(path.c_str(), &buf);
         if (0 != ret)                    return false;
         if (ct && !S_ISREG(buf.st_mode)) return false;
         return true;
     } // fileExists
 
+#ifdef EXV_UNICODE_PATH
+    bool fileExists(const std::wstring& wpath, bool ct)
+    {
+        struct _stat buf;
+        int ret = _wstat(wpath.c_str(), &buf);
+        if (0 != ret)                    return false;
+        if (ct && !S_ISREG(buf.st_mode)) return false;
+        return true;
+    } // fileExists
+
+#endif
     std::string strError()
     {
         int error = errno;
         std::ostringstream os;
 #ifdef EXV_HAVE_STRERROR_R
         const size_t n = 1024;
-# ifdef EXV_STRERROR_R_CHAR_P
+// _GNU_SOURCE: See Debian bug #485135
+# if defined EXV_STRERROR_R_CHAR_P || defined _GNU_SOURCE
         char *buf = 0;
         char buf2[n];
         std::memset(buf2, 0x0, n);
@@ -95,7 +107,7 @@ namespace Exiv2 {
 #else
         os << std::strerror(error);
 #endif
-        os << " (" << error << ")";
+        os << " (errno = " << error << ")";
         return os.str();
     } // strError
 
