@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      pentaxmn.cpp
-  Version:   $Rev: 1532 $
+  Version:   $Rev: 1937 $
   Author(s): Michal Cihar <michal@cihar.com>
   Based on fujimn.cpp by:
              Andreas Huggel (ahu) <ahuggel@gmx.net>
@@ -30,7 +30,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: pentaxmn.cpp 1532 2008-07-16 04:38:32Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: pentaxmn.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -610,7 +610,10 @@ namespace Exiv2 {
         { 0x07f2, "smc PENTAX-DA* 16-50mm F2.8 ED AL [IF] SDM (SDM not used)" },
         { 0x07f3, "smc PENTAX-DA 70mm F2.4 Limited" },
         { 0x07f4, "smc PENTAX-DA 21mm F3.2 AL Limited" },
-        { 0x08ea, "smc PENTAX-DA* 300 mm F4ED [IF] SDM" },
+        { 0x08e2, "smc PENTAX-DA* 55mm F1.4 SDM" },
+        { 0x08e3, "smc PENTAX DA* 60-250mm F4 [IF] SDM"},
+        { 0x08e8, "smc PENTAX-DA 17-70mm F4 AL [IF] SDM" },
+        { 0x08ea, "smc PENTAX-DA* 300mm F4 ED [IF] SDM" },
         { 0x08eb, "smc PENTAX-DA* 200mm F2.8 ED [IF] SDM" },
         { 0x08f1, "smc PENTAX-DA* 50-135mm F2.8 ED [IF] SDM" },
         { 0x08f2, "smc PENTAX-DA* 16-50mm F2.8 ED AL [IF] SDM" },
@@ -740,7 +743,7 @@ namespace Exiv2 {
                << " EV";
         }
 
-        if (value.size() == 2) {
+        if (value.count() == 2) {
             long l1 = value.toLong(1);
             long type;
             long range;
