@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -23,7 +23,7 @@
   @brief   PNG image, implemented using the following references:
            <a href="http://www.w3.org/TR/PNG/">PNG specification</a> by W3C<br>
            <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/PNG.html">PNG tags list</a> by Phil Harvey<br>
-  @version $Rev: 1703 $
+  @version $Rev: 1937 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Gilles Caulier (cgilles)
@@ -78,7 +78,6 @@ namespace Exiv2
               method to get a temporary reference.
           @param create Specifies if an existing image should be read (false)
               or if a new file should be created (true).
-              This option is not yet implemented.
          */
         PngImage(BasicIo::AutoPtr io, bool create);
         //@}
@@ -91,7 +90,7 @@ namespace Exiv2
 
         //! @name Accessors
         //@{
-        std::string mimeType() const { return "image/png"; }
+        std::string mimeType() const;
         //@}
 
     private:
