@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      preview.cpp
-  Version:   $Rev: 1704 $
+  Version:   $Rev: 1937 $
   Author(s): Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   18-Sep-08, vn: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: preview.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: preview.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -247,6 +247,7 @@ namespace {
         { 0,                   createLoaderExifDataJpeg, 4 },
         { 0,                   createLoaderExifDataJpeg, 5 },
         { 0,                   createLoaderExifDataJpeg, 6 },
+        { "image/x-raw",       createLoaderExifDataJpeg, 7 },
         { 0,                   createLoaderTiff,         0 },
         { 0,                   createLoaderTiff,         1 },
         { 0,                   createLoaderTiff,         2 },
@@ -281,7 +282,8 @@ namespace {
         { "Exif.Minolta.ThumbnailOffset",            "Exif.Minolta.ThumbnailLength"                  }, // 3
         { "Exif.Olympus.ThumbnailImage",             0                                               }, // 4
         { "Exif.Olympus2.ThumbnailImage",            0                                               }, // 5
-        { "Exif.Minolta.Thumbnail",                  0                                               }  // 6
+        { "Exif.Minolta.Thumbnail",                  0                                               }, // 6
+        { "Exif.PanasonicRaw.PreviewImage",          0                                               }  // 7
     };
 
     const LoaderTiff::Param LoaderTiff::param_[] = {
@@ -370,6 +372,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/jpeg";
         prop.extension_ = ".jpg";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".jpg");
+#endif
         return prop;
     }
 
@@ -383,7 +388,7 @@ namespace {
         }
         IoCloser closer(io);
 
-        const byte *base = io.mmap();
+        const Exiv2::byte* base = io.mmap();
 
         return DataBuf(base + offset_, size_);
     }
@@ -399,7 +404,7 @@ namespace {
             throw Error(9, io.path(), strError());
         }
         IoCloser closer(io);
-        const byte *base = io.mmap();
+        const Exiv2::byte* base = io.mmap();
 
         try {
             Image::AutoPtr image = ImageFactory::open(base + offset_, size_);
@@ -445,6 +450,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/jpeg";
         prop.extension_ = ".jpg";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".jpg");
+#endif
         return prop;
     }
 
@@ -554,6 +562,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/tiff";
         prop.extension_ = ".tif";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".tif");
+#endif
         return prop;
     }
 
@@ -608,7 +619,7 @@ namespace {
             }
             IoCloser closer(io);
 
-            const byte *base = io.mmap();
+            const Exiv2::byte* base = io.mmap();
 
             const Value &sizes = preview["Exif.Image." + sizeTag_].value();
 
@@ -622,7 +633,7 @@ namespace {
             else {
                 // FIXME: the buffer is probably copied twice, it should be optimized
                 DataBuf buf(size_);
-                byte *pos = buf.pData_;
+                Exiv2::byte* pos = buf.pData_;
                 for (int i = 0; i < sizes.count(); i++) {
                     uint32_t offset = dataValue.toLong(i);
                     uint32_t size = sizes.toLong(i);
@@ -635,11 +646,11 @@ namespace {
         }
 
         // write new image
-        Blob blob;
+        MemIo mio;
         const IptcData emptyIptc;
         const XmpData  emptyXmp;
-        TiffParser::encode(blob, 0, 0, Exiv2::littleEndian, preview, emptyIptc, emptyXmp);
-        return DataBuf((blob.size() > 0 ? &blob[0] : 0), static_cast<long>(blob.size()));
+        TiffParser::encode(mio, 0, 0, Exiv2::littleEndian, preview, emptyIptc, emptyXmp);
+        return DataBuf(mio.mmap(), mio.size());
     }
 
 }                                       // namespace
@@ -690,6 +701,16 @@ namespace Exiv2 {
         return Exiv2::writeFile(buf, name);
     }
 
+#ifdef EXV_UNICODE_PATH
+    long PreviewImage::writeFile(const std::wstring& wpath) const
+    {
+        std::wstring name = wpath + wextension();
+        // Todo: Creating a DataBuf here unnecessarily copies the memory
+        DataBuf buf(pData_, size_);
+        return Exiv2::writeFile(buf, name);
+    }
+
+#endif
     DataBuf PreviewImage::copy() const
     {
         return DataBuf(pData_, size_);
@@ -715,6 +736,13 @@ namespace Exiv2 {
         return properties_.extension_;
     }
 
+#ifdef EXV_UNICODE_PATH
+    std::wstring PreviewImage::wextension() const
+    {
+        return properties_.wextension_;
+    }
+
+#endif
     uint32_t PreviewImage::width() const
     {
         return properties_.width_;
