@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      rafimage.cpp
-  Version:   $Rev: 1620 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   05-Feb-07, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: rafimage.cpp 1620 2008-10-09 14:10:51Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: rafimage.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -59,6 +59,11 @@ namespace Exiv2 {
     {
     } // RafImage::RafImage
 
+    std::string RafImage::mimeType() const
+    {
+        return "image/x-fuji-raf";
+    }
+
     int RafImage::pixelWidth() const
     {
         Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
