@@ -3,7 +3,7 @@
   Abstract:  Print a simple comma separated list of tags defined in Exiv2
 
   File:      taglist.cpp
-  Version:   $Rev: 1512 $
+  Version:   $Rev: 1740 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   07-Jan-04, ahu: created
  */
@@ -38,7 +38,7 @@ try {
         }
 
         IfdId ifdId = ExifTags::ifdIdByIfdItem(item);
-        if (ExifTags::isMakerIfd(ifdId)) {
+        if (ExifTags::isExifIfd(ifdId) || ExifTags::isMakerIfd(ifdId)) {
             ExifTags::taglist(std::cout, ifdId);
             break;
         }
