@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version $Rev: 1680 $
+  @version $Rev: 1960 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -66,7 +66,8 @@ namespace Exiv2 {
     enum SectionId { sectionIdNotSet,
                      imgStruct, recOffset, imgCharacter, otherTags, exifFormat,
                      exifVersion, imgConfig, userInfo, relatedFile, dateTime,
-                     captureCond, gpsTags, iopTags, makerTags, dngTags,
+                     captureCond, gpsTags, iopTags, makerTags, dngTags, panaRaw,
+                     tiffEp,
                      lastSectionId };
 
 // *****************************************************************************
@@ -140,6 +141,24 @@ namespace Exiv2 {
         const char* label_;                     //!< Description of the tag value
     }; // struct TagDetailsBitmask
 
+    /*!
+      @brief Helper structure for lookup tables for translations of controlled
+             vocabulary strings to their descriptions.
+     */
+    struct EXIV2API TagVocabulary {
+        const char* voc_;                       //!< Vocabulary string
+        const char* label_;                     //!< Description of the vocabulary string
+
+        /*!
+          @brief Comparison operator for use with the find template
+
+          Compare vocabulary strings like "PR-NON" with keys like
+          "http://ns.useplus.org/ldf/vocab/PR-NON" and return true if the vocabulary
+          string matches the end of the key.
+         */
+        bool operator==(const std::string& key) const;
+    }; // struct TagDetails
+
     /*!
       @brief Generic pretty-print function to translate a long value to a description
              by looking up a reference table.
@@ -173,7 +192,7 @@ namespace Exiv2 {
             if (td->mask_ == 0) return os << exvGettext(td->label_);
         }
         bool sep = false;
-        for (int i = 0; i < N; i++) {
+        for (int i = 0; i < N; ++i) {
             // *& acrobatics is a workaround for a MSVC 7.1 bug
             const TagDetailsBitmask* td = *(&array) + i;
 
@@ -193,6 +212,26 @@ namespace Exiv2 {
 //! Shortcut for the printTagBitmask template which requires typing the array name only once.
 #define EXV_PRINT_TAG_BITMASK(array) printTagBitmask<EXV_COUNTOF(array), array>
 
+    /*!
+      @brief Generic pretty-print function to translate a controlled vocabulary value (string)
+             to a description by looking up a reference table.
+     */
+    template <int N, const TagVocabulary (&array)[N]>
+    std::ostream& printTagVocabulary(std::ostream& os, const Value& value, const ExifData*)
+    {
+        const TagVocabulary* td = find(array, value.toString());
+        if (td) {
+            os << exvGettext(td->label_);
+        }
+        else {
+            os << "(" << value << ")";
+        }
+        return os;
+    }
+
+//! Shortcut for the printTagVocabulary template which requires typing the array name only once.
+#define EXV_PRINT_VOCABULARY(array) printTagVocabulary<EXV_COUNTOF(array), array>
+
     //! Exif tag reference, implemented as a static class.
     class EXIV2API ExifTags {
         //! Prevent construction: not implemented.
@@ -368,16 +407,16 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
-        virtual std::string key() const { return key_; }
-        virtual const char* familyName() const { return familyName_; }
+        virtual std::string key() const;
+        virtual const char* familyName() const;
         /*!
           @brief Return the name of the group (the second part of the key).
                  For Exif keys, the group name is the IFD item.
         */
-        virtual std::string groupName() const { return ifdItem(); }
+        virtual std::string groupName() const;
         virtual std::string tagName() const;
         virtual std::string tagLabel() const;
-        virtual uint16_t tag() const { return tag_; }
+        virtual uint16_t tag() const;
 
         AutoPtr clone() const;
         //! Return the IFD id
