@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      tgaimage.cpp
-  Version:   $Rev: 1703 $
+  Version:   $Rev: 1937 $
   Author(s): Marco Piovanelli, Ovolab (marco)
   History:   05-Mar-2007, marco: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: tgaimage.cpp 1703 2008-12-15 16:24:37Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: tgaimage.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 //#define DEBUG 1
 
@@ -57,6 +57,11 @@ namespace Exiv2 {
     {
     } // TgaImage::TgaImage
 
+    std::string TgaImage::mimeType() const
+    {
+        return "image/targa";
+    }
+
     void TgaImage::setExifData(const ExifData& /*exifData*/)
     {
         // Todo: implement me!
@@ -144,12 +149,19 @@ namespace Exiv2 {
     bool isTgaType(BasicIo& iIo, bool /*advance*/)
     {
         // not all TARGA files have a signature string, so first just try to match the file name extension
-        std::string path = iIo.path();
-        if(path.rfind(".tga") != std::string::npos || path.rfind(".TGA") != std::string::npos)
-        {
+#ifdef EXV_UNICODE_PATH
+        std::wstring wpath = iIo.wpath();
+        if(   wpath.rfind(EXV_WIDEN(".tga")) != std::wstring::npos
+           || wpath.rfind(EXV_WIDEN(".TGA")) != std::wstring::npos) {
             return true;
         }
-
+#else
+        std::string path = iIo.path();
+        if(   path.rfind(".tga") != std::string::npos
+           || path.rfind(".TGA") != std::string::npos) {
+            return true;
+        }
+#endif
         byte buf[26];
         long curPos = iIo.tell();
         iIo.seek(-26, BasicIo::end);
