@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      tiffvisitor.cpp
-  Version:   $Rev: 1702 $
+  Version:   $Rev: 1977 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   11-Apr-06, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 1702 2008-12-15 15:16:33Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -56,10 +56,10 @@ EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 1702 2008-12-15 15:16:33Z ahuggel $")
 // *****************************************************************************
 namespace {
     //! Unary predicate that matches an Exifdatum with a given group and index.
-    class FindExifdatum {
+    class FindExifdatum2 {
     public:
         //! Constructor, initializes the object with the group and index to look for.
-        FindExifdatum(uint16_t group, int idx)
+        FindExifdatum2(uint16_t group, int idx)
             : groupName_(Exiv2::Internal::tiffGroupName(group)), idx_(idx) {}
         //! Returns true if group and index match.
         bool operator()(const Exiv2::Exifdatum& md) const
@@ -71,7 +71,16 @@ namespace {
         const char* groupName_;
         int idx_;
 
-    }; // class FindExifdatum
+    }; // class FindExifdatum2
+
+    Exiv2::ByteOrder stringToByteOrder(const std::string& val)
+    {
+        Exiv2::ByteOrder bo = Exiv2::invalidByteOrder;
+        if (0 == strcmp("II", val.c_str())) bo = Exiv2::littleEndian;
+        else if (0 == strcmp("MM", val.c_str())) bo = Exiv2::bigEndian;
+
+        return bo;
+    }
 }
 
 // *****************************************************************************
@@ -86,6 +95,10 @@ namespace Exiv2 {
         }
     }
 
+    TiffVisitor::~TiffVisitor()
+    {
+    }
+
     void TiffVisitor::setGo(GoEvent event, bool go)
     {
         assert(event >= 0 && event < events_);
@@ -98,11 +111,32 @@ namespace Exiv2 {
         return go_[event];
     }
 
+    void TiffVisitor::visitDirectoryNext(TiffDirectory* /*object*/)
+    {
+    }
+
+    void TiffVisitor::visitDirectoryEnd(TiffDirectory* /*object*/)
+    {
+    }
+
+    void TiffVisitor::visitIfdMakernoteEnd(TiffIfdMakernote* /*object*/)
+    {
+    }
+
+    void TiffVisitor::visitBinaryArrayEnd(TiffBinaryArray* /*object*/)
+    {
+    }
+
     void TiffFinder::init(uint16_t tag, uint16_t group)
     {
         tag_ = tag;
         group_ = group;
         tiffComponent_ = 0;
+        setGo(geTraverse, true);
+    }
+
+    TiffFinder::~TiffFinder()
+    {
     }
 
     void TiffFinder::findObject(TiffComponent* object)
@@ -153,12 +187,12 @@ namespace Exiv2 {
         findObject(object);
     }
 
-    void TiffFinder::visitArrayEntry(TiffArrayEntry* object)
+    void TiffFinder::visitBinaryArray(TiffBinaryArray* object)
     {
         findObject(object);
     }
 
-    void TiffFinder::visitArrayElement(TiffArrayElement* object)
+    void TiffFinder::visitBinaryElement(TiffBinaryElement* object)
     {
         findObject(object);
     }
@@ -192,6 +226,10 @@ namespace Exiv2 {
         }
     }
 
+    TiffDecoder::~TiffDecoder()
+    {
+    }
+
     void TiffDecoder::visitEntry(TiffEntry* object)
     {
         decodeTiffEntry(object);
@@ -364,7 +402,7 @@ namespace Exiv2 {
 
     void TiffDecoder::decodeStdTiffEntry(const TiffEntryBase* object)
     {
-        assert(object !=0);
+        assert(object != 0);
         // Todo: ExifKey should have an appropriate c'tor, it should not be
         //       necessary to use groupName here
         ExifKey key(object->tag(), tiffGroupName(object->group()));
@@ -373,12 +411,14 @@ namespace Exiv2 {
 
     } // TiffDecoder::decodeTiffEntry
 
-    void TiffDecoder::visitArrayEntry(TiffArrayEntry* /*object*/)
+    void TiffDecoder::visitBinaryArray(TiffBinaryArray* object)
     {
-        // Nothing to do
+        if (object->cfg() == 0) {
+            decodeTiffEntry(object);
+        }
     }
 
-    void TiffDecoder::visitArrayElement(TiffArrayElement* object)
+    void TiffDecoder::visitBinaryElement(TiffBinaryElement* object)
     {
         decodeTiffEntry(object);
     }
@@ -424,6 +464,10 @@ namespace Exiv2 {
         }
     }
 
+    TiffEncoder::~TiffEncoder()
+    {
+    }
+
     void TiffEncoder::encodeIptc()
     {
         // Update IPTCNAA Exif tag, if it exists. Delete the tag if there
@@ -446,7 +490,17 @@ namespace Exiv2 {
         }
         if (rawIptc.size_ != 0 && (del || pos == exifData_.end())) {
             Value::AutoPtr value = Value::create(unsignedLong);
-            value->read(rawIptc.pData_, rawIptc.size_, byteOrder_);
+            DataBuf buf;
+            if (rawIptc.size_ % 4 != 0) {
+                // Pad the last unsignedLong value with 0s
+                buf.alloc((rawIptc.size_ / 4) * 4 + 4);
+                memset(buf.pData_, 0x0, buf.size_);
+                memcpy(buf.pData_, rawIptc.pData_, rawIptc.size_);
+            }
+            else {
+                buf = rawIptc; // Note: This resets rawIptc
+            }
+            value->read(buf.pData_, buf.size_, byteOrder_);
             Exifdatum iptcDatum(iptcNaaKey, value.get());
             exifData_.add(iptcDatum);
             pos = exifData_.findKey(irbKey); // needed after add()
@@ -459,7 +513,7 @@ namespace Exiv2 {
             irbBuf = Photoshop::setIptcIrb(irbBuf.pData_, irbBuf.size_, iptcData_);
             exifData_.erase(pos);
             if (irbBuf.size_ != 0) {
-                Value::AutoPtr value = Value::create(undefined);
+                Value::AutoPtr value = Value::create(unsignedByte);
                 value->read(irbBuf.pData_, irbBuf.size_, invalidByteOrder);
                 Exifdatum iptcDatum(irbKey, value.get());
                 exifData_.add(iptcDatum);
@@ -587,11 +641,21 @@ namespace Exiv2 {
     void TiffEncoder::visitIfdMakernote(TiffIfdMakernote* object)
     {
         assert(object != 0);
+
+        ExifData::iterator pos = exifData_.findKey(ExifKey("Exif.MakerNote.ByteOrder"));
+        if (pos != exifData_.end()) {
+            // Set Makernote byte order
+            ByteOrder bo = stringToByteOrder(pos->toString());
+            if (bo != invalidByteOrder && bo != object->byteOrder()) {
+                object->setByteOrder(bo);
+                setDirty();
+            }
+            if (del_) exifData_.erase(pos);
+        }
         if (del_) {
-            // Remove synthesized tags
+            // Remove remaining synthesized tags
             static const char* synthesizedTags[] = {
                 "Exif.MakerNote.Offset",
-                "Exif.MakerNote.ByteOrder",
             };
             for (unsigned int i = 0; i < EXV_COUNTOF(synthesizedTags); ++i) {
                 ExifData::iterator pos = exifData_.findKey(ExifKey(synthesizedTags[i]));
@@ -599,9 +663,8 @@ namespace Exiv2 {
             }
         }
         // Modify encoder for Makernote peculiarities, byte order
-        if (object->byteOrder() != invalidByteOrder) {
-            byteOrder_ = object->byteOrder();
-        }
+        byteOrder_ = object->byteOrder();
+
     } // TiffEncoder::visitIfdMakernote
 
     void TiffEncoder::visitIfdMakernoteEnd(TiffIfdMakernote* /*object*/)
@@ -611,14 +674,41 @@ namespace Exiv2 {
 
     } // TiffEncoder::visitIfdMakernoteEnd
 
-    void TiffEncoder::visitArrayEntry(TiffArrayEntry* /*object*/)
+    void TiffEncoder::visitBinaryArray(TiffBinaryArray* /*object*/)
     {
         // Nothing to do
     }
 
-    void TiffEncoder::visitArrayElement(TiffArrayElement* object)
+    void TiffEncoder::visitBinaryArrayEnd(TiffBinaryArray* object)
     {
+        assert(object != 0);
+
+        int32_t size = object->TiffEntryBase::doSize();
+        if (size == 0) return;
+        if (!object->initialize(pRoot_)) return;
+
+        // Re-encrypt buffer if necessary
+        const CryptFct cryptFct = object->cfg()->cryptFct_;
+        if (cryptFct != 0) {
+            const byte* pData = object->pData();
+            DataBuf buf = cryptFct(object->tag(), pData, size, pRoot_);
+            if (buf.size_ > 0) {
+                pData = buf.pData_;
+                size = buf.size_;
+            }
+            if (!object->updOrigDataBuf(pData, size)) {
+                setDirty();
+            }
+        }
+    }
+
+    void TiffEncoder::visitBinaryElement(TiffBinaryElement* object)
+    {
+        // Temporarily overwrite byteorder according to that of the binary element
+        ByteOrder boOrig = byteOrder_;
+        if (object->elByteOrder() != invalidByteOrder) byteOrder_ = object->elByteOrder();
         encodeTiffComponent(object);
+        byteOrder_ = boOrig;
     }
 
     void TiffEncoder::encodeTiffComponent(
@@ -640,7 +730,7 @@ namespace Exiv2 {
                     // Try to find exact match (in case of duplicate tags)
                     ExifData::iterator pos2 =
                         std::find_if(exifData_.begin(), exifData_.end(),
-                                     FindExifdatum(object->group(), object->idx()));
+                                     FindExifdatum2(object->group(), object->idx()));
                     if (pos2 != exifData_.end() && pos2->key() == key.key()) {
                         ed = &(*pos2);
                         pos = pos2; // make sure we delete the correct tag below
@@ -679,16 +769,16 @@ namespace Exiv2 {
 #endif
     } // TiffEncoder::encodeTiffComponent
 
-    void TiffEncoder::encodeArrayElement(TiffArrayElement* object, const Exifdatum* datum)
+    void TiffEncoder::encodeBinaryArray(TiffBinaryArray* object, const Exifdatum* datum)
+    {
+        encodeOffsetEntry(object, datum);
+    } // TiffEncoder::encodeBinaryArray
+
+    void TiffEncoder::encodeBinaryElement(TiffBinaryElement* object, const Exifdatum* datum)
     {
         encodeTiffEntryBase(object, datum);
     } // TiffEncoder::encodeArrayElement
 
-    void TiffEncoder::encodeArrayEntry(TiffArrayEntry* object, const Exifdatum* datum)
-    {
-        encodeOffsetEntry(object, datum);
-    } // TiffEncoder::encodeArrayEntry
-
     void TiffEncoder::encodeDataEntry(TiffDataEntry* object, const Exifdatum* datum)
     {
         encodeOffsetEntry(object, datum);
@@ -865,17 +955,10 @@ namespace Exiv2 {
 
     } // TiffEncoder::encodeOffsetEntry
 
-    void TiffEncoder::encodeBigEndianEntry(TiffEntryBase* object, const Exifdatum* datum)
-    {
-        byteOrder_ = bigEndian;
-        encodeTiffEntryBase(object, datum);
-        byteOrder_ = origByteOrder_;
-    }
-
     void TiffEncoder::add(
-        TiffComponent*     pRootDir,
-        TiffComponent*     pSourceDir,
-        TiffCompFactoryFct createFct
+        TiffComponent* pRootDir,
+        TiffComponent* pSourceDir,
+        uint32_t       root
     )
     {
         assert(pRootDir != 0);
@@ -886,21 +969,23 @@ namespace Exiv2 {
         // iterate over all remaining entries.
         del_ = false;
 
+        ExifData::const_iterator posBo = exifData_.end();
         for (ExifData::const_iterator i = exifData_.begin();
              i != exifData_.end(); ++i) {
 
             uint16_t group = tiffGroupId(i->groupName());
             // Skip synthesized info tags
-            if (group == Group::mn) continue;
+            if (group == Group::mn) {
+                if (i->tag() == 0x0002) {
+                    posBo = i;
+                }
+                continue;
+            }
 
             // Assumption is that the corresponding TIFF entry doesn't exist
-
-            // Todo: getPath depends on the Creator class, not the createFct
-            //       how to get it through to here???
-
             TiffPath tiffPath;
-            TiffCreator::getPath(tiffPath, i->tag(), group);
-            TiffComponent* tc = pRootDir->addPath(i->tag(), tiffPath);
+            TiffCreator::getPath(tiffPath, i->tag(), group, root);
+            TiffComponent* tc = pRootDir->addPath(i->tag(), tiffPath, pRootDir);
             TiffEntryBase* object = dynamic_cast<TiffEntryBase*>(tc);
 #ifdef DEBUG
             if (object == 0) {
@@ -914,128 +999,30 @@ namespace Exiv2 {
                 encodeTiffComponent(object, &(*i));
             }
         }
+
+        /*
+          What follows is a hack. I can't think of a better way to set
+          the makernote byte order (and other properties maybe) in the
+          makernote header during intrusive writing. The thing is that
+          visit/encodeIfdMakernote is not called in this case and there
+          can't be an Exif tag which corresponds to this component.
+         */
+        if (posBo == exifData_.end()) return;
+
+        TiffFinder finder(0x927c, Group::exif);
+        pRootDir->accept(finder);
+        TiffMnEntry* te = dynamic_cast<TiffMnEntry*>(finder.result());
+        if (te) {
+            TiffIfdMakernote* tim = dynamic_cast<TiffIfdMakernote*>(te->mn_);
+            if (tim) {
+                // Set Makernote byte order
+                ByteOrder bo = stringToByteOrder(posBo->toString());
+                if (bo != invalidByteOrder) tim->setByteOrder(bo);
+            }
+        }
+
     } // TiffEncoder::add
 
-    const std::string TiffPrinter::indent_("   ");
-
-    void TiffPrinter::incIndent()
-    {
-        prefix_ += indent_;
-    } // TiffPrinter::incIndent
-
-    void TiffPrinter::decIndent()
-    {
-        if (prefix_.length() >= indent_.length()) {
-            prefix_.erase(prefix_.length() - indent_.length(), indent_.length());
-        }
-    } // TiffPrinter::decIndent
-
-    void TiffPrinter::visitEntry(TiffEntry* object)
-    {
-        printTiffEntry(object, prefix());
-    } // TiffPrinter::visitEntry
-
-    void TiffPrinter::visitDataEntry(TiffDataEntry* object)
-    {
-        printTiffEntry(object, prefix());
-        if (object->pValue()) {
-            os_ << prefix() << _("Data area") << " "
-                << object->pValue()->sizeDataArea()
-                << " " << _("bytes.\n");
-        }
-    } // TiffPrinter::visitDataEntry
-
-    void TiffPrinter::visitImageEntry(TiffImageEntry* object)
-    {
-        printTiffEntry(object, prefix());
-    } // TiffPrinter::visitImageEntry
-
-    void TiffPrinter::visitSizeEntry(TiffSizeEntry* object)
-    {
-        printTiffEntry(object, prefix());
-    }
-
-    void TiffPrinter::visitDirectory(TiffDirectory* object)
-    {
-        assert(object != 0);
-        os_ << prefix() << tiffGroupName(object->group())
-            << " " << _("directory with") << " "
-            // cast to make MSVC happy
-            << std::dec << static_cast<unsigned int>(object->components_.size());
-        if (object->components_.size() == 1) os_ << " " << _("entry:\n");
-        else os_ << " " << _("entries:\n");
-        incIndent();
-
-    } // TiffPrinter::visitDirectory
-
-    void TiffPrinter::visitDirectoryNext(TiffDirectory* object)
-    {
-        decIndent();
-        if (object->hasNext()) {
-            if (object->pNext_) os_ << prefix() << _("Next directory:\n");
-            else os_ << prefix() << _("No next directory\n");
-        }
-    } // TiffPrinter::visitDirectoryNext
-
-    void TiffPrinter::visitDirectoryEnd(TiffDirectory* /*object*/)
-    {
-        // Nothing to do
-    } // TiffPrinter::visitDirectoryEnd
-
-    void TiffPrinter::visitSubIfd(TiffSubIfd* object)
-    {
-        os_ << prefix() << _("Sub-IFD") << " ";
-        printTiffEntry(object);
-    } // TiffPrinter::visitSubIfd
-
-    void TiffPrinter::visitMnEntry(TiffMnEntry* object)
-    {
-        if (!object->mn_) printTiffEntry(object, prefix());
-        else os_ << prefix() << _("Makernote") << " ";
-    } // TiffPrinter::visitMnEntry
-
-    void TiffPrinter::visitIfdMakernote(TiffIfdMakernote* /*object*/)
-    {
-        // Nothing to do
-    } // TiffPrinter::visitIfdMakernote
-
-    void TiffPrinter::printTiffEntry(TiffEntryBase* object,
-                                     const std::string& px) const
-    {
-        assert(object != 0);
-
-        os_ << px << tiffGroupName(object->group())
-            << " " << _("tag") << " 0x" << std::setw(4) << std::setfill('0')
-            << std::hex << std::right << object->tag()
-            << ", " << _("type") << " 0x" << std::hex << object->tiffType()
-            << ", " << std::dec << object->count() << " "<< _("component");
-        if (object->count() > 1) os_ << "s";
-        os_ << " in " << object->size() << " " << _("bytes");
-        if (object->size() > 4) os_ << ", " << _("offset") << " " << object->offset();
-        os_ << "\n";
-        const Value* vp = object->pValue();
-        if (vp && vp->count() < 100) os_ << prefix() << *vp;
-        else os_ << prefix() << "...";
-        os_ << "\n";
-
-    } // TiffPrinter::printTiffEntry
-
-    void TiffPrinter::visitArrayEntry(TiffArrayEntry* object)
-    {
-        os_ << prefix() << _("Array Entry") << " " << tiffGroupName(object->group())
-            << " " << _("tag") << " 0x" << std::setw(4) << std::setfill('0')
-            << std::hex << std::right << object->tag() << " " << _("with")
-            << " " << std::dec << object->count() << " ";
-        if (object->count() > 1) os_ << _("elements");
-        else os_ << _("element");
-        os_ << "\n";
-    } // TiffPrinter::visitArrayEntry
-
-    void TiffPrinter::visitArrayElement(TiffArrayElement* object)
-    {
-        printTiffEntry(object, prefix());
-    } // TiffPrinter::visitArrayElement
-
     TiffReader::TiffReader(const byte*    pData,
                            uint32_t       size,
                            TiffComponent* pRoot,
@@ -1045,7 +1032,8 @@ namespace Exiv2 {
           pLast_(pData + size),
           pRoot_(pRoot),
           pState_(state.release()),
-          pOrigState_(pState_)
+          pOrigState_(pState_),
+          postProc_(false)
     {
         assert(pData_);
         assert(size_ > 0);
@@ -1067,8 +1055,6 @@ namespace Exiv2 {
     {
         if (state.get() != 0) {
             if (pOrigState_ != pState_) delete pState_;
-            // 0 for create function indicates 'no change'
-            if (state->createFct_ == 0) state->createFct_ = pState_->createFct_;
             // invalidByteOrder indicates 'no change'
             if (state->byteOrder_ == invalidByteOrder) state->byteOrder_ = pState_->byteOrder_;
             pState_ = state.release();
@@ -1087,14 +1073,6 @@ namespace Exiv2 {
         return pState_->baseOffset_;
     }
 
-    TiffComponent::AutoPtr TiffReader::create(uint32_t extendedTag,
-                                              uint16_t group) const
-    {
-        assert(pState_);
-        assert(pState_->createFct_);
-        return pState_->createFct_(extendedTag, group);
-    }
-
     void TiffReader::readDataEntryBase(TiffDataEntryBase* object)
     {
         assert(object != 0);
@@ -1156,6 +1134,15 @@ namespace Exiv2 {
         return ++idxSeq_[group];
     }
 
+    void TiffReader::postProcess()
+    {
+        postProc_ = true;
+        for (PostList::const_iterator pos = postList_.begin(); pos != postList_.end(); ++pos) {
+            (*pos)->accept(*this);
+        }
+        postProc_ = false;
+    }
+
     void TiffReader::visitDirectory(TiffDirectory* object)
     {
         assert(object != 0);
@@ -1195,7 +1182,7 @@ namespace Exiv2 {
                 return;
             }
             uint16_t tag = getUShort(p, byteOrder());
-            TiffComponent::AutoPtr tc = create(tag, object->group());
+            TiffComponent::AutoPtr tc = TiffCreator::create(tag, object->group());
             // The assertion typically fails if a component is not configured in
             // the TIFF structure table
             assert(tc.get());
@@ -1216,7 +1203,7 @@ namespace Exiv2 {
             TiffComponent::AutoPtr tc(0);
             uint32_t next = getLong(p, byteOrder());
             if (next) {
-                tc = create(Tag::next, object->group());
+                tc = TiffCreator::create(Tag::next, object->group());
 #ifndef SUPPRESS_WARNINGS
                 if (tc.get() == 0) {
                     std::cerr << "Warning: "
@@ -1310,6 +1297,8 @@ namespace Exiv2 {
     {
         assert(object != 0);
 
+        object->setImageByteOrder(byteOrder()); // set the byte order for the image
+
         if (!object->readHeader(object->start(),
                                 static_cast<uint32_t>(pLast_ - object->start()),
                                 byteOrder())) {
@@ -1336,8 +1325,6 @@ namespace Exiv2 {
         );
         changeState(state);
 
-        object->byteOrder_ = byteOrder(); // set the actual byte order of the makernote
-
     } // TiffReader::visitIfdMakernote
 
     void TiffReader::visitIfdMakernoteEnd(TiffIfdMakernote* /*object*/)
@@ -1391,9 +1378,11 @@ namespace Exiv2 {
         }
         p += 4;
         uint32_t size = typeSize * count;
-        uint32_t offset = getLong(p, byteOrder());
+        int32_t offset = getLong(p, byteOrder());
         byte* pData = p;
-        if (size > 4 && baseOffset() + offset >= size_) {
+        if (   size > 4
+            && (   baseOffset() + offset >= size_
+                || static_cast<int32_t>(baseOffset()) + offset < 0)) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Error: Offset of "
                           << "directory " << tiffGroupName(object->group())
@@ -1438,62 +1427,57 @@ namespace Exiv2 {
 
     } // TiffReader::readTiffEntry
 
-    void TiffReader::visitArrayEntry(TiffArrayEntry* object)
+    void TiffReader::visitBinaryArray(TiffBinaryArray* object)
     {
         assert(object != 0);
 
-        readTiffEntry(object);
-        // Todo: size here is that of the data area
-        const uint16_t sz = static_cast<uint16_t>(object->size_ / object->elSize());
-        for (uint16_t i = 0; i < sz; ++i) {
-            uint16_t tag = i;
-            TiffComponent::AutoPtr tc = create(tag, object->elGroup());
-            assert(tc.get());
-            tc->setStart(object->pData() + i * object->elSize());
-            object->addChild(tc);
-            // Hack: Exif.CanonCs.Lens has 3 components
-            if (object->elGroup() == Group::canoncs && tag == 0x0017) {
-                i += 2;
-            }
-        }
-
-    } // TiffReader::visitArrayEntry
-
-    void TiffReader::visitArrayElement(TiffArrayElement* object)
-    {
-        assert(object != 0);
-
-        TypeId typeId = toTypeId(object->elTiffType(), object->tag(), object->group());
-        uint32_t size = TypeInfo::typeSize(typeId);
-        // Hack: Exif.CanonCs.Lens has 3 components
-        if (object->group() == Group::canoncs && object->tag() == 0x0017) {
-            size *= 3;
-        }
-        byte* pData   = object->start();
-        assert(pData >= pData_);
-
-        if (pData + size > pLast_) {
-#ifndef SUPPRESS_WARNINGS
-            std::cerr << "Error: Array element in group "
-                      << tiffGroupName(object->group())
-                      << "requests access to memory beyond the data buffer. "
-                      << "Skipping element.\n";
-#endif
+        if (!postProc_) {
+            // Defer reading children until after all other components are read, but
+            // since state (offset) is not set during post-processing, read entry here
+            readTiffEntry(object);
+            object->iniOrigDataBuf();
+            postList_.push_back(object);
             return;
         }
 
+        if (object->TiffEntryBase::doSize() == 0) return;
+        if (!object->initialize(pRoot_)) return;
+        const ArrayDef* defs = object->def();
+        const ArrayDef* def = &object->cfg()->elDefaultDef_;
+
+        const CryptFct cryptFct = object->cfg()->cryptFct_;
+        if (cryptFct != 0) {
+            const byte* pData = object->pData();
+            int32_t size = object->TiffEntryBase::doSize();
+            DataBuf buf = cryptFct(object->tag(), pData, size, pRoot_);
+            if (buf.size_ > 0) object->setData(buf);
+        }
+
+        for (uint32_t idx = 0; idx < object->TiffEntryBase::doSize(); ) {
+            if (defs) {
+                def = std::find(defs, defs + object->defSize(), idx);
+                if (def == defs + object->defSize()) def = &object->cfg()->elDefaultDef_;
+            }
+            idx += object->addElement(idx, def); // idx may be different from def->idx_
+        }
+
+    } // TiffReader::visitBinaryArray
+
+    void TiffReader::visitBinaryElement(TiffBinaryElement* object)
+    {
+        byte* pData   = object->start();
+        uint32_t size = object->TiffEntryBase::doSize();
         ByteOrder bo = object->elByteOrder();
         if (bo == invalidByteOrder) bo = byteOrder();
+        TypeId typeId = toTypeId(object->elDef()->tiffType_, object->tag(), object->group());
         Value::AutoPtr v = Value::create(typeId);
         assert(v.get());
         v->read(pData, size, bo);
 
         object->setValue(v);
-        object->setData(pData, size);
         object->setOffset(0);
         object->setIdx(nextIdx(object->group()));
-        object->setCount(1);
 
-    } // TiffReader::visitArrayElement
+    } // TiffReader::visitBinaryElement
 
 }}                                      // namespace Internal, Exiv2
