@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      types.cpp
-  Version:   $Rev: 1599 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: types.cpp 1599 2008-09-19 15:26:57Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: types.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -35,6 +35,9 @@ EXIV2_RCSID("@(#) $Id: types.cpp 1599 2008-09-19 15:26:57Z ahuggel $")
 #include "i18n.h"                               // for _exvGettext
 
 // + standard includes
+#ifdef EXV_UNICODE_PATH
+# include <windows.h> // for MultiByteToWideChar etc
+#endif
 #include <string>
 #include <iostream>
 #include <iomanip>
@@ -375,6 +378,32 @@ namespace Exiv2 {
 #endif
     }
 
+#ifdef EXV_UNICODE_PATH
+    std::string ws2s(const std::wstring& s)
+    {
+        int len;
+        int slength = (int)s.length() + 1;
+        len = WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, 0, 0, 0, 0);
+        char* buf = new char[len];
+        WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, buf, len, 0, 0);
+        std::string r(buf);
+        delete[] buf;
+        return r;
+    }
+
+    std::wstring s2ws(const std::string& s)
+    {
+        int len;
+        int slength = (int)s.length() + 1;
+        len = MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, 0, 0);
+        wchar_t* buf = new wchar_t[len];
+        MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, buf, len);
+        std::wstring r(buf);
+        delete[] buf;
+        return r;
+    }
+
+#endif // EXV_UNICODE_PATH
     template<>
     bool stringTo<bool>(const std::string& s, bool& ok)
     {
