@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    types.hpp
   @brief   Type definitions for %Exiv2 and related functionality
-  @version $Rev: 1689 $
+  @version $Rev: 1983 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created<BR>
@@ -52,6 +52,11 @@
 # include <stdint.h>
 #endif
 
+// MSVC macro to convert a string to a wide string
+#ifdef EXV_UNICODE_PATH
+# define EXV_WIDEN(t) L ## t
+#endif
+
 // MSVC doesn't provide C99 types, but it has MS specific variants
 #ifdef _MSC_VER
 typedef unsigned __int8  uint8_t;
@@ -68,6 +73,12 @@ typedef __int32          int32_t;
  */
 #define EXV_CALL_MEMBER_FN(object,ptrToMember)  ((object).*(ptrToMember))
 
+// Simple min and max macros
+//! Simple common min macro
+#define EXV_MIN(a,b) ((a) < (b) ? (a) : (b))
+//! Simple common max macro
+#define EXV_MAX(a,b) ((a) > (b) ? (a) : (b))
+
 // *****************************************************************************
 // forward declarations
 struct tm;
@@ -134,23 +145,82 @@ namespace Exiv2 {
     };
 
     //! Type to specify the IFD to which a metadata belongs
-    enum IfdId { ifdIdNotSet,
-                 ifd0Id, exifIfdId, gpsIfdId, iopIfdId, ifd1Id, ifd2Id,
-                 subImage1Id, subImage2Id, subImage3Id, subImage4Id,
-                 mnIfdId,
-                 canonIfdId, canonCsIfdId, canonSiIfdId, canonCfIfdId,
-                 canonPiIfdId, canonPaIfdId,
-                 fujiIfdId,
-                 minoltaIfdId, minoltaCs5DIfdId, minoltaCs7DIfdId,
-                 minoltaCsOldIfdId, minoltaCsNewIfdId,
-                 nikon1IfdId, nikon2IfdId, nikon3IfdId, nikonPvIfdId,
-                 olympusIfdId, olympus2IfdId, olympusCsIfdId, olympusEqIfdId,
-                 olympusRdIfdId, olympusRd2IfdId, olympusIpIfdId, olympusFiIfdId,
-                 olympusFe1IfdId, olympusFe2IfdId, olympusFe3IfdId, olympusFe4IfdId,
-                 olympusFe5IfdId, olympusFe6IfdId, olympusFe7IfdId, olympusFe8IfdId,
-                 olympusFe9IfdId, olympusRiIfdId,
-                 panasonicIfdId, pentaxIfdId, sigmaIfdId, sonyIfdId,
-                 lastIfdId };
+    enum IfdId {
+        ifdIdNotSet,
+        ifd0Id,
+        exifIfdId,
+        gpsIfdId,
+        iopIfdId,
+        ifd1Id,
+        ifd2Id,
+        ifd3Id,
+        subImage1Id,
+        subImage2Id,
+        subImage3Id,
+        subImage4Id,
+        mnIfdId,
+        canonIfdId,
+        canonCsIfdId,
+        canonSiIfdId,
+        canonCfIfdId,
+        canonPiIfdId,
+        canonPaIfdId,
+        canonFiIfdId,
+        fujiIfdId,
+        minoltaIfdId,
+        minoltaCs5DIfdId,
+        minoltaCs7DIfdId,
+        minoltaCsOldIfdId,
+        minoltaCsNewIfdId,
+        nikon1IfdId,
+        nikon2IfdId,
+        nikon3IfdId,
+        nikonPvIfdId,
+        nikonVrIfdId,
+        nikonPcIfdId,
+        nikonWtIfdId,
+        nikonIiIfdId,
+        nikonAfIfdId,
+        nikonSi1IfdId,
+        nikonSi2IfdId,
+        nikonSi3IfdId,
+        nikonSi4IfdId,
+        nikonSi5IfdId,
+        nikonSi6IfdId,
+        nikonLd1IfdId,
+        nikonLd2IfdId,
+        nikonLd3IfdId,
+        nikonCb1IfdId,
+        nikonCb2IfdId,
+        nikonCb2aIfdId,
+        nikonCb2bIfdId,
+        nikonCb3IfdId,
+        nikonCb4IfdId,
+        olympusIfdId,
+        olympus2IfdId,
+        olympusCsIfdId,
+        olympusEqIfdId,
+        olympusRdIfdId,
+        olympusRd2IfdId,
+        olympusIpIfdId,
+        olympusFiIfdId,
+        olympusFe1IfdId,
+        olympusFe2IfdId,
+        olympusFe3IfdId,
+        olympusFe4IfdId,
+        olympusFe5IfdId,
+        olympusFe6IfdId,
+        olympusFe7IfdId,
+        olympusFe8IfdId,
+        olympusFe9IfdId,
+        olympusRiIfdId,
+        panasonicIfdId,
+        panaRawIfdId,
+        pentaxIfdId,
+        sigmaIfdId,
+        sonyIfdId,
+        lastIfdId
+    };
 
     //! Container for binary data
     typedef std::vector<byte> Blob;
@@ -168,7 +238,7 @@ namespace Exiv2 {
         TypeInfo& operator=(const TypeInfo& rhs);
 
     public:
-        //! Return the name of the type
+        //! Return the name of the type, 0 if unknown.
         static const char* typeName(TypeId typeId);
         //! Return the type id for a type name
         static TypeId typeId(const std::string& typeName);
@@ -345,6 +415,13 @@ namespace Exiv2 {
      */
     EXIV2API const char* exvGettext(const char* str);
 
+#ifdef EXV_UNICODE_PATH
+    //! Convert an std::string s to a unicode string returned as a std::wstring.
+    EXIV2API std::wstring s2ws(const std::string& s);
+    //! Convert a unicode std::wstring s to an std::string.
+    EXIV2API std::string ws2s(const std::wstring& s);
+
+#endif
     /*!
       @brief Return a \em long set to the value represented by \em s.
 
