@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      utils.cpp
-  Version:   $Rev: 1366 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: utils.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: utils.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -61,6 +61,15 @@ namespace Util {
 
 // *****************************************************************************
 // class Getopt
+    Getopt::Getopt()
+        : errcnt_(0)
+    {
+    }
+
+    Getopt::~Getopt()
+    {
+    }
+
     int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
     {
         progname_ = Util::basename(argv[0]);
