@@ -1,6 +1,6 @@
 // ********************************************************* -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    utils.hpp
   @brief   A collection of utility functions
-  @version $Rev: 1366 $
+  @version $Rev: 1937 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    12-Dec-03, ahu: created
@@ -58,10 +58,10 @@ namespace Util {
 class Getopt {
 public:
     //! Default constructor.
-    Getopt() : errcnt_(0) {}
+    Getopt();
 
     //! Destructor.
-    virtual ~Getopt() {}
+    virtual ~Getopt();
 
     /*!
       @brief Parse command line arguments.
