@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      value.cpp
-  Version:   $Rev: 1673 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: value.cpp 1673 2008-11-13 12:28:20Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: value.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -152,6 +152,16 @@ namespace Exiv2 {
         return toString();
     }
 
+    long Value::sizeDataArea() const
+    {
+        return 0;
+    }
+
+    DataBuf Value::dataArea() const
+    {
+        return DataBuf(0, 0);
+    };
+
     DataValue::DataValue(TypeId typeId) : Value(typeId)
     {
     }
@@ -167,6 +177,11 @@ namespace Exiv2 {
     {
     }
 
+    long DataValue::count() const
+    {
+        return size();
+    }
+
     int DataValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // byteOrder not needed
@@ -209,7 +224,8 @@ namespace Exiv2 {
     {
         std::vector<byte>::size_type end = value_.size();
         for (std::vector<byte>::size_type i = 0; i != end; ++i) {
-            os << static_cast<int>(value_[i]) << " ";
+            os << static_cast<int>(value_[i]);
+            if (i < end - 1) os << " ";
         }
         return os;
     }
@@ -240,6 +256,26 @@ namespace Exiv2 {
         return Rational(value_[n], 1);
     }
 
+    StringValueBase::StringValueBase(TypeId typeId)
+        : Value(typeId)
+    {
+    }
+
+    StringValueBase::StringValueBase(TypeId typeId, const std::string& buf)
+        : Value(typeId)
+    {
+        read(buf);
+    }
+
+    StringValueBase::StringValueBase(const StringValueBase& rhs)
+        : Value(rhs), value_(rhs.value_)
+    {
+    }
+
+    StringValueBase::~StringValueBase()
+    {
+    }
+
     StringValueBase& StringValueBase::operator=(const StringValueBase& rhs)
     {
         if (this == &rhs) return *this;
@@ -271,6 +307,11 @@ namespace Exiv2 {
             );
     }
 
+    long StringValueBase::count() const
+    {
+        return size();
+    }
+
     long StringValueBase::size() const
     {
         return static_cast<long>(value_.size());
@@ -299,11 +340,39 @@ namespace Exiv2 {
         return Rational(value_[n], 1);
     }
 
+    StringValue::StringValue()
+        : StringValueBase(string)
+    {
+    }
+
+    StringValue::StringValue(const std::string& buf)
+        : StringValueBase(string, buf)
+    {
+    }
+
+    StringValue::~StringValue()
+    {
+    }
+
     StringValue* StringValue::clone_() const
     {
         return new StringValue(*this);
     }
 
+    AsciiValue::AsciiValue()
+        : StringValueBase(asciiString)
+    {
+    }
+
+    AsciiValue::AsciiValue(const std::string& buf)
+        : StringValueBase(asciiString, buf)
+    {
+    }
+
+    AsciiValue::~AsciiValue()
+    {
+    }
+
     int AsciiValue::read(const std::string& buf)
     {
         value_ = buf;
@@ -370,12 +439,21 @@ namespace Exiv2 {
                invalidCharsetId : charsetTable_[i].charsetId_;
     }
 
+    CommentValue::CommentValue()
+        : StringValueBase(Exiv2::undefined)
+    {
+    }
+
     CommentValue::CommentValue(const std::string& comment)
         : StringValueBase(Exiv2::undefined)
     {
         read(comment);
     }
 
+    CommentValue::~CommentValue()
+    {
+    }
+
     int CommentValue::read(const std::string& comment)
     {
         std::string c = comment;
@@ -483,7 +561,7 @@ namespace Exiv2 {
         std::ostringstream os;
         write(os);
         std::string s = os.str();
-        std::memcpy(buf, &s[0], s.size());
+        if (s.size() > 0) std::memcpy(buf, &s[0], s.size());
         return static_cast<long>(s.size());
     }
 
@@ -772,6 +850,10 @@ namespace Exiv2 {
         date_.day = day;
     }
 
+    DateValue::~DateValue()
+    {
+    }
+
     int DateValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // Hard coded to read Iptc style dates
@@ -834,6 +916,16 @@ namespace Exiv2 {
         return 8;
     }
 
+    const DateValue::Date& DateValue::getDate() const
+    {
+        return date_;
+    }
+
+    long DateValue::count() const
+    {
+        return size();
+    }
+
     long DateValue::size() const
     {
         return 8;
@@ -865,6 +957,16 @@ namespace Exiv2 {
         return l;
     }
 
+    float DateValue::toFloat(long n) const
+    {
+        return static_cast<float>(toLong(n));
+    }
+
+    Rational DateValue::toRational(long n) const
+    {
+        return Rational(toLong(n), 1);
+    }
+
     TimeValue::TimeValue()
         : Value(time)
     {
@@ -882,6 +984,10 @@ namespace Exiv2 {
         time_.tzMinute = tzMinute;
     }
 
+    TimeValue::~TimeValue()
+    {
+    }
+
     int TimeValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // Make the buffer a 0 terminated C-string for scanTime[36]
@@ -985,6 +1091,16 @@ namespace Exiv2 {
         return 11;
     }
 
+    const TimeValue::Time& TimeValue::getTime() const
+    {
+        return time_;
+    }
+
+    long TimeValue::count() const
+    {
+        return size();
+    }
+
     long TimeValue::size() const
     {
         return 11;
@@ -1021,4 +1137,14 @@ namespace Exiv2 {
         return result;
     }
 
+    float TimeValue::toFloat(long n) const
+    {
+        return static_cast<float>(toLong(n));
+    }
+
+    Rational TimeValue::toRational(long n) const
+    {
+        return Rational(toLong(n), 1);
+    }
+
 }                                       // namespace Exiv2
