@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      xmp.cpp
-  Version:   $Rev: 1703 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   13-July-07, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: xmp.cpp 1703 2008-12-15 16:24:37Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: xmp.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -203,6 +203,11 @@ namespace Exiv2 {
         return TypeInfo::typeName(typeId());
     }
 
+    long Xmpdatum::typeSize() const
+    {
+        return 0;
+    }
+
     long Xmpdatum::count() const
     {
         return p_->value_.get() == 0 ? 0 : p_->value_->count();
@@ -377,7 +382,9 @@ namespace Exiv2 {
 #ifdef EXV_HAVE_XMP_TOOLKIT
             initialized_ = SXMPMeta::Initialize();
             SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/1.0/", "digiKam", 0);
-            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/Photo/1.0/", "MicrosoftPhoto", 0);
+            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/photo/1.0/", "MicrosoftPhoto", 0);
+            SXMPMeta::RegisterNamespace("http://iptc.org/std/Iptc4xmpExt/2008-02-29/", "iptcExt", 0);
+            SXMPMeta::RegisterNamespace("http://ns.useplus.org/ldf/xmp/1.0/", "plus", 0);
 #else
             initialized_ = true;
 #endif
