@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      xmpsidecar.cpp
-  Version:   $Rev: 1690 $
+  Version:   $Rev: 1937 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   07-Mar-08, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: xmpsidecar.cpp 1690 2008-12-07 10:51:51Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: xmpsidecar.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -68,6 +68,11 @@ namespace Exiv2 {
         }
     } // XmpSidecar::XmpSidecar
 
+    std::string XmpSidecar::mimeType() const
+    {
+        return "application/rdf+xml";
+    }
+
     void XmpSidecar::setComment(const std::string& /*comment*/)
     {
         // not supported
